﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;

namespace Common.Unit
{
    /// <summary>SQL执行的相关帮助类</summary>
    public class SqlUnit
    {
        /// <summary>创建分页SQL【pageIndex=当前页，pageSize=每页，sql=查询sql（如果有内聚的需要加 select top），rowBy=生成row的列名】</summary>
        public static string CreatePagerSql(int pageIndex, int pageSize, string sql, string rowBy)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendFormat(" SELECT TOP ({0}) * FROM ( SELECT top(1000) ROW_NUMBER() OVER (ORDER BY a.{1} desc) AS id,* FROM ( ", pageSize.ToString(), rowBy);
            sb.Append(sql);
            sb.AppendFormat(") as a  )as t WHERE t.id > ({0}*({1}-1)) order by t.id", pageSize.ToString(), pageIndex.ToString());
            return sb.ToString();
        }
        /// <summary>DataTable的一行数据转为对象以方便JSON序列化 【dt=DataTable数据源 rowIndex=需要转换的row】</summary>
        public static Dictionary<string, string> RowDataToDict(DataTable dt, int rowIndex)
        {
            Dictionary<string, string> list = new Dictionary<string, string>();
            if (dt.Rows.Count > rowIndex)
            {
                DataColumnCollection columns = dt.Columns;
                DataRow row = dt.Rows[rowIndex];
                foreach (var one in columns)
                {
                    string columnName = one.ToString();
                    string data = row[columnName] is DBNull ? "" : row[columnName].ToString();
                    list.Add(columnName, data);
                }
            }
            else
            {
                DataColumnCollection columns = dt.Columns;
                foreach (var one in columns)
                {
                    string columnName = one.ToString();
                    list.Add(columnName, "");
                }
            }
            return list;
        }
        /// <summary>获得dt第一行某一列的数据【dt=数据源 column=某列】</summary>
        public static string GetValue(DataTable dt, string column)
        {
            string value = dt.Rows[0][column] is DBNull ? "" : dt.Rows[0][column].ToString();
            return value;
        }
        /// <summary>获得dt第某行某一列的数据【dt=数据源 column=某列 i=某行】</summary>
        public static string GetValue(DataTable dt, string column, int i)
        {
            string value = dt.Rows[i][column] is DBNull ? "" : dt.Rows[i][column].ToString();
            return value;
        }
        /// <summary>获得dt第某行某一列的数据【dt=数据源 column=某列 i=某行】</summary>
        public static string GetValue(DataRow dr, string column)
        {
            string value = dr[column] is DBNull ? "" : dr[column].ToString();
            return value;
        }

        /// <summary>组装add sql</summary>
        public static string MakeAddSql(string tableName, Dictionary<string, string> column)
        {
            StringBuilder sb = new StringBuilder();
            string columnName = "";
            string val = ""; 
            sb.Append(" insert into " + tableName + " ( ");
            foreach (var one in column)
            {
                columnName += "[" + one.Key + "],";
                val += "'" + one.Value + "',";
            }
            columnName = columnName.Substring(0, columnName.Length - 1);
            val = val.Substring(0, val.Length - 1);
            sb.Append(columnName);
            sb.Append(" ) values ( ");
            sb.Append(val);
            sb.Append(" );");
            return sb.ToString();
        }

        /// <summary>组装更新SQL</summary>
        public static string MakeUptSql(string tableName, Dictionary<string, string> column, Dictionary<string, string> where)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append(" update  " + tableName + " set ");
            string setSql = string.Empty;
            foreach (var one in column)
            {
                setSql += one.Key + "='" + one.Value + "',";
            }
            setSql = setSql.Substring(0, setSql.Length - 1);
            sb.Append(setSql);
            string whereSql = string.Empty;
            foreach (var one in where)
            {
                whereSql += one.Key + "='" + one.Value + "' and";
            }
            if (!string.IsNullOrEmpty(whereSql))
            {
                sb.Append(" where ");
                whereSql = whereSql + " 1=1";
                sb.Append(whereSql);
            }
            sb.Append(";");
            return sb.ToString();
        }
        public static string MakeTransSql(List<string> sqlList)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append(" begin tran; ");
            foreach (string one in sqlList)
            {
                sb.Append(one + "; ");
            }
            sb.Append(" commit tran;  ");
            return sb.ToString();
        }

    }
}


