﻿using Common.Unit;
using G.MES.API.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web;
using System.Web.Http.Controllers;
using System.Web.Http.Filters;

namespace G.MES.API.App_Start
{
    /// <summary>
    /// 接口 token 校验
    /// </summary>
    public class AuthFilterAttribute : AuthorizationFilterAttribute
    {
        public override void OnAuthorization(HttpActionContext actionContext)
        {
            string enable = System.Configuration.ConfigurationManager.AppSettings["EnableToken"].ToString();
            if (enable != "1") { return; }
            if (actionContext.ActionDescriptor.GetCustomAttributes<NoAuthAttribute>()
                    .Any()
                || actionContext.ActionDescriptor.ControllerDescriptor
                    .GetCustomAttributes<NoAuthAttribute>().Any())
            {
                return;

            }

            var authHeader = from t in actionContext.Request.Headers where t.Key == "api_key" select t.Value.FirstOrDefault();
            var headerkey = authHeader == null ? null : authHeader.FirstOrDefault();
            var parakey = "";
            if (HttpContext.Current!=null)
            {
                parakey= HttpContext.Current.Request.QueryString["api_key"] == null ? "" : HttpContext.Current.Request.QueryString["api_key"];

            }
            string checktoken = string.IsNullOrEmpty(parakey) ? headerkey : parakey;
            string errinfo = "";
            if (string.IsNullOrEmpty(checktoken)) { errinfo = "未检测到Token，请登陆"; }
            else
            {
                errinfo = CheckToken(checktoken, actionContext);
            }
            if (!string.IsNullOrEmpty(errinfo))
            {
                var result = new ApiResult()
                {
                    Status = "ERR",
                    StatusCode = HttpStatusCode.BadRequest,
                    MsgCode = 10001,
                    Msg = errinfo
                };
                actionContext.Response = actionContext.Request.CreateResponse(HttpStatusCode.BadRequest, result);
            }
        }
        private string CheckToken(string token, HttpActionContext actionContext)
        {
            var info = JWTToken.Decode(token);
            if (info == null) return "Token验证失败，请登陆";
            actionContext.RequestContext.RouteData.Values.Add("LoginUid", info.Uid);
            return "";
        }
    }
}