﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using Common.Unit;
using G.Buss.System;
using Newtonsoft.Json;
using G.Buss.BaseInfo;
using G.MES.DataModel;
using Common.Unit.Redis;
using G.MES.ViewModel;
using G.MES.API.App_Start;

namespace G.MES.API.Controllers
{


    public class LoginController : BasicController
    {
        private LoginSvc loginSvc = new LoginSvc();
        /// <summary>
        /// 登录接口
        /// </summary>
        /// <param name="uid">登录账号</param>
        /// <param name="pwd">密码</param>
        /// <param name="VerCodeID">验证码ID</param>
        /// <param name="VerCode">验证码</param>
        /// <returns></returns>
        [NoAuthAttribute]
        [HttpGet]
        public LoginInfo Login(string uid, string pwd, string VerCodeID = "", string VerCode = "")
        {
            if (string.IsNullOrEmpty(uid))
            {
                returnWarn("用户名不能为空");
            }
            RedisHelper redis = new RedisHelper();

            var EnablePassWordStrength =
                System.Configuration.ConfigurationManager.AppSettings["EnablePassWordStrength"] == null ? "" : System.Configuration.ConfigurationManager.AppSettings["EnablePassWordStrength"].ToString();
            //启用了密码策略
            if (EnablePassWordStrength == "1")
            {
                if (string.IsNullOrEmpty(VerCodeID))
                {
                    returnWarn("验证码ID不能为空");
                }
                if (string.IsNullOrEmpty(VerCode))
                {
                    returnWarn("验证码不能为空");
                }

                var exists = redis.StringGet<string>(VerCodeID);
                if (string.IsNullOrEmpty(exists))
                    returnWarn("验证码过期");
                if (exists.ToLower() != VerCode.ToLower())
                {
                    returnWarn("验证码不正确");
                }
                redis.KeyDelete(VerCodeID);
            }



            var info = loginSvc.UserLogin(uid, UPEncry.EncryString(pwd));
            if (info.flag != true)
            {
                returnWarn(info.messages);
            }
            LoginInfo loginInfo = (LoginInfo)info.data;
            if (info != null)
            {
                //生成token
                loginInfo.Token = JWTToken.Encode(uid);
                //redis的操作
                try
                {
                    var exist = redis.StringGet<LoginInfo>(uid);
                    if (exist != null)
                        redis.ListRemove(uid, exist);
                }
                catch { }
                redis.StringSet(uid, loginInfo);
            }
            return loginInfo;
        }


        ///// <summary>
        ///// 登录接口
        ///// </summary>
        ///// <param name="uid">登录账号</param>
        ///// <param name="pwd">密码</param>
        ///// <returns></returns>
        //[HttpGet]
        //public LoginInfo Login(string uid, string pwd)
        //{
        //    if (string.IsNullOrEmpty(uid))
        //    {
        //        returnWarn("用户名不能为空");
        //    }
        //    var info = loginSvc.UserLogin(uid, UPEncry.EncryString(pwd));
        //    if (info.flag != true)
        //    {
        //        returnWarn(info.messages);
        //    }
        //    LoginInfo loginInfo = (LoginInfo)info.data;
        //    if (info != null)
        //    {
        //        //生成token



        //        loginInfo.Token = JWTToken.Encode(uid);
        //        //redis的操作
        //        RedisHelper redis = new RedisHelper();
        //        try
        //        {
        //            var exist = redis.StringGet<LoginInfo>(uid);
        //            if (exist != null)
        //                redis.ListRemove(uid, exist);
        //        }
        //        catch { }
        //        redis.StringSet(uid, loginInfo);
        //    }
        //    return loginInfo;
        //}
    }
}
