﻿using Aspose.Cells;
using Chloe;
using Chloe.SqlServer;
using Common.Unit;
using G.Buss.BaseInfo.ViewModel.ClientInfoModel;
using G.MES.DataModel;
using G.MES.ViewModel;
using NPOI.HSSF.UserModel;
using NPOI.SS.UserModel;
using NPOI.XSSF.UserModel;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Web;

namespace G.Buss.BaseInfo.Services.ClientInfoSVC
{
    public class ClientInfoSVC : BaseSvc
    {
        #region 获取详情

        public List<ClientInfoDetailModel> DoGetClientInfoDetail(List<Guid> requestModel)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    List<ClientInfoDetailModel> thisQueryModel = context.Query<ClientInfo>().Where(a => requestModel.Contains(a.ClientInfoOID))
                        .Select(a => new ClientInfoDetailModel
                        {
                            ClientInfoOID = a.ClientInfoOID,
                            ItemsName = a.ItemsName,
                            ItemsType = a.ItemsType,
                            ExperimentalType = a.ExperimentalType,
                            Name = a.Name,
                            YKSampleNum = a.YKSampleNum,
                            RummageNum = a.RummageNum,
                            ClientSampleName = a.ClientSampleName,
                            Entity = a.Entity,
                            Sex = a.Sex,
                            ReceivedDate = a.ReceivedDate,
                            ProjectDate = a.ProjectDate,
                            NodeTitleDate = a.NodeTitleDate,
                            GroupingSituation = a.GroupingSituation,
                            Batch = a.Batch,
                            CaptureProbe = a.CaptureProbe,
                            Extracter = a.Extracter,
                            QualityPeople = a.QualityPeople,
                            LabOperator = a.LabOperator,
                            DeliveryDate = a.DeliveryDate,
                            Index_I5 = a.Index_I5,
                            Index_I7 = a.Index_I7,
                            SpotCheck = a.SpotCheck,
                            QualityManagement = a.QualityManagement,
                            ProcessMode = a.ProcessMode,
                            ClientDataSize = a.ClientDataSize,
                            DeplaneDataSize = a.DeplaneDataSize,
                            AdditionalNum = a.AdditionalNum,
                            FaultRemark = a.FaultRemark,
                            ClientUnit = a.ClientUnit,
                            Email = a.Email,
                            Phone = a.Phone,
                            ProjectNumber = a.ProjectNumber,
                            Birth = a.Birth,
                            IDCard = a.IDCard,
                            SampleDetection = a.SampleDetection,
                            TechnologyDetection = a.TechnologyDetection,
                            DetectingPlatform = a.DetectingPlatform,
                            TNM = a.TNM,
                            IsEssential = a.IsEssential,
                            ClinicalDiagnosis = a.ClinicalDiagnosis,
                            FamilyHistory = a.FamilyHistory,
                            PersonalHistory = a.PersonalHistory
                            //SampleCode = a.SampleCode,
                            //SampleType = a.SampleType,
                            //TumorsType = a.TumorsType,
                            //SampleSource = a.SampleSource,
                        }).ToList();
                    thisQueryModel[0].ClientDataSizeList = GetClientDataSizeCodeValue();

                    return thisQueryModel;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion
        public List<CodeValueIDAndValueModel> GetClientDataSizeCodeValue()
        {
            using (var context = new MsSqlContext(sqlConn))
            {
                List<CodeValueIDAndValueModel> AllSonDictionaryDirectory = context.Query<CodeValue>()
                    .Where(a => a.CodeTableID == "ClientDataSize")
                    .Select(a => new CodeValueIDAndValueModel
                    {
                        CodeID = a.CodeID,
                        CodeName = a.CodeName,
                    }).ToList();
                return AllSonDictionaryDirectory;
            }
        }


        #region 获取列表（带查）
        public PageData DoGetClientInfo(ClientInfoRequestModel requestModel, LoginInfo user)
        {
            try
            {
                GetWhere(requestModel, user, out LamadaExtention<ClientInfoQueryModel> where);
                using (var context = new MsSqlContext(sqlConn))
                {
                    var ClientInfoQuery = context.JoinQuery<ClientInfo, Sys_Organize>((ClientInfo, org) => new object[] {
                        JoinType.LeftJoin,ClientInfo.SYS_ORG == org.Sys_OrganizeOID
                    }).Select((ClientInfo, org) => new ClientInfoQueryModel
                    {
                        ClientInfoOID = ClientInfo.ClientInfoOID,
                        Name = ClientInfo.Name,
                        Sex = ClientInfo.Sex,
                        IDCard = ClientInfo.IDCard,
                        Birth = ClientInfo.Birth,
                        SampleCode = ClientInfo.SampleCode,
                        ClassCode = org.ClassCode
                    }).Where(where.GetLambda());
                    int TotalNum = ClientInfoQuery.Count();

                    List<ClientInfoQueryModel> ClientInfoList = ClientInfoQuery.OrderBy(a => a.Name).TakePage(requestModel.pageindex, requestModel.pagesize).ToList();

                    return PageData.Make(requestModel.pageindex, requestModel.pagesize, TotalNum, ClientInfoList);
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }


        public PageData DoGetAllClientInfo(ClientInfoRequestModel requestModel)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    var ClientInfoQuery = context.JoinQuery<ClientInfo, Sys_Organize>((ClientInfo, org) => new object[] {
                        JoinType.LeftJoin,ClientInfo.SYS_ORG == org.Sys_OrganizeOID
                    }).Select((ClientInfo, org) => new ClientInfoQueryModel
                    {
                        ClientInfoOID = ClientInfo.ClientInfoOID,
                        Name = ClientInfo.Name,
                        Sex = ClientInfo.Sex,
                        IDCard = ClientInfo.IDCard,
                        Birth = ClientInfo.Birth,
                        SampleCode = ClientInfo.SampleCode,
                        ClassCode = org.ClassCode
                    });
                    int TotalNum = ClientInfoQuery.Count();
                    List<ClientInfoQueryModel> ClientInfoList = ClientInfoQuery.OrderBy(a => a.Name).TakePage(requestModel.pageindex, requestModel.pagesize).ToList();
                    return PageData.Make(requestModel.pageindex, requestModel.pagesize, TotalNum, ClientInfoList);
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        /// <summary>
        /// 获取筛选条件
        /// </summary>
        /// <param name="requestModel"></param>
        /// <param name="user"></param>
        /// <returns></returns>
        public void GetWhere(ClientInfoRequestModel requestModel, LoginInfo user, out LamadaExtention<ClientInfoQueryModel> where)
        {
            try
            {
                where = new LamadaExtention<ClientInfoQueryModel>();
                if (!string.IsNullOrEmpty(requestModel.Name))
                {
                    where.And(a => a.Name.Contains(requestModel.Name));
                }
                if (requestModel.Sex != null)
                {
                    where.And(a => a.Sex == requestModel.Sex);
                }
                if (!string.IsNullOrEmpty(requestModel.IDCard))
                {
                    where.And(a => a.IDCard.Contains(requestModel.IDCard));
                }
                if (requestModel.Birth != null)
                {
                    if (!string.IsNullOrEmpty(requestModel.Birth.MaxValue))
                    {
                        where.And(a => a.Birth < Convert.ToDateTime(requestModel.Birth.MaxValue));
                    }
                    if (!string.IsNullOrEmpty(requestModel.Birth.MinValue))
                    {
                        where.And(a => a.Birth > Convert.ToDateTime(requestModel.Birth.MinValue));
                    }
                }
                if (!string.IsNullOrEmpty(requestModel.SampleCode))
                {
                    where.And(a => a.SampleCode.Contains(requestModel.SampleCode));
                }
                if (user.Sys_UserOID != new Guid() && user.Sys_UserOID != new Guid("00000000-0000-0000-0000-000000000001"))
                {
                    where.And(a => a.ClassCode.StartsWith(user.OrgClassCode));
                }
            }
            catch (Exception ex)
            {
                throw new Exception("获取筛选条件时：" + ex.Message);
            }

        }
        #endregion


        #region 保存（带更新）
        public bool DoSaveClientInfo(ClientInfoSaveRequestModel ClientInfo, LoginInfo user)
        {
            try
            {
                if (ClientInfo.ClientInfoOID == Guid.Empty)
                {
                    ClientInfo.ClientInfoOID = Guid.NewGuid();
                }
                using (var context = new MsSqlContext(sqlConn))
                {
                    ClientInfo nowSam = context.Query<ClientInfo>().Where(a => a.ClientInfoOID == ClientInfo.ClientInfoOID).FirstOrDefault();
                    if (nowSam != null)
                    {
                        //更新
                        context.TrackEntity(nowSam);
                        nowSam.Sex = ClientInfo.Sex;
                        nowSam.Name = ClientInfo.Name;
                        nowSam.IDCard = ClientInfo.IDCard;
                        nowSam.Birth = ClientInfo.Birth;
                        nowSam.SampleCode = ClientInfo.SampleCode;
                        nowSam.SampleType = ClientInfo.SampleType;
                        nowSam.Phone = ClientInfo.Phone;
                        nowSam.SampleDetection = ClientInfo.SampleDetection;
                        nowSam.TechnologyDetection = ClientInfo.TechnologyDetection;
                        nowSam.TumorsType = ClientInfo.TumorsType;
                        nowSam.TNM = ClientInfo.TNM;
                        nowSam.IsEssential = ClientInfo.IsEssential;
                        nowSam.ClinicalDiagnosis = ClientInfo.ClinicalDiagnosis;
                        nowSam.FamilyHistory = ClientInfo.FamilyHistory;
                        nowSam.PersonalHistory = ClientInfo.PersonalHistory;
                        nowSam.SamplingTime = ClientInfo.SamplingTime;
                        nowSam.ItemsName = ClientInfo.ItemsName;
                        nowSam.ItemsType = ClientInfo.ItemsType;
                        nowSam.ExperimentalType = ClientInfo.ExperimentalType;
                        nowSam.YKSampleNum = ClientInfo.YKSampleNum;
                        nowSam.RummageNum = ClientInfo.RummageNum;
                        nowSam.ClientSampleName = ClientInfo.ClientSampleName;
                        nowSam.Entity = ClientInfo.Entity;
                        nowSam.ReceivedDate = ClientInfo.ReceivedDate;
                        nowSam.ProjectDate = ClientInfo.ProjectDate;
                        nowSam.NodeTitleDate = ClientInfo.NodeTitleDate;
                        nowSam.SampleSource = ClientInfo.SampleSource;
                        nowSam.GroupingSituation = ClientInfo.GroupingSituation;
                        nowSam.Batch = ClientInfo.Batch;
                        nowSam.CaptureProbe = ClientInfo.CaptureProbe;
                        nowSam.Extracter = ClientInfo.Extracter;
                        nowSam.QualityPeople = ClientInfo.QualityPeople;
                        nowSam.LabOperator = ClientInfo.LabOperator;
                        nowSam.DeliveryDate = ClientInfo.DeliveryDate;
                        nowSam.Index_I5 = ClientInfo.Index_I5;
                        nowSam.Index_I7 = ClientInfo.Index_I7;
                        nowSam.SpotCheck = ClientInfo.SpotCheck;
                        nowSam.QualityManagement = ClientInfo.QualityManagement;
                        nowSam.ProcessMode = ClientInfo.ProcessMode;
                        nowSam.ClientDataSize = ClientInfo.ClientDataSize;
                        nowSam.DeplaneDataSize = ClientInfo.DeplaneDataSize;
                        nowSam.AdditionalNum = ClientInfo.AdditionalNum;
                        nowSam.FaultRemark = ClientInfo.FaultRemark;
                        nowSam.ClientUnit = ClientInfo.ClientUnit;
                        nowSam.Email = ClientInfo.Email;
                        nowSam.ProjectNumber = ClientInfo.ProjectNumber;
                        nowSam.DetectingPlatform = ClientInfo.DetectingPlatform;
                        UpSysField(nowSam, user);
                        context.Update(nowSam);
                    }
                    else
                    {
                        //新增
                        nowSam = new ClientInfo();
                        nowSam.ClientInfoOID = ClientInfo.ClientInfoOID;
                        nowSam.Sex = ClientInfo.Sex;
                        nowSam.Name = ClientInfo.Name;
                        nowSam.IDCard = ClientInfo.IDCard;
                        nowSam.Birth = ClientInfo.Birth;
                        nowSam.SampleCode = ClientInfo.SampleCode;
                        nowSam.SampleType = ClientInfo.SampleType;
                        nowSam.Phone = ClientInfo.Phone;
                        nowSam.SampleDetection = ClientInfo.SampleDetection;
                        nowSam.TechnologyDetection = ClientInfo.TechnologyDetection;
                        nowSam.TumorsType = ClientInfo.TumorsType;
                        nowSam.TNM = ClientInfo.TNM;
                        nowSam.IsEssential = ClientInfo.IsEssential;
                        nowSam.ClinicalDiagnosis = ClientInfo.ClinicalDiagnosis;
                        nowSam.FamilyHistory = ClientInfo.FamilyHistory;
                        nowSam.PersonalHistory = ClientInfo.PersonalHistory;
                        nowSam.SamplingTime = ClientInfo.SamplingTime;
                        nowSam.ItemsName = ClientInfo.ItemsName;
                        nowSam.ItemsType = ClientInfo.ItemsType;
                        nowSam.ExperimentalType = ClientInfo.ExperimentalType;
                        nowSam.YKSampleNum = ClientInfo.YKSampleNum;
                        nowSam.RummageNum = ClientInfo.RummageNum;
                        nowSam.ClientSampleName = ClientInfo.ClientSampleName;
                        nowSam.Entity = ClientInfo.Entity;
                        nowSam.ReceivedDate = ClientInfo.ReceivedDate;
                        nowSam.ProjectDate = ClientInfo.ProjectDate;
                        nowSam.NodeTitleDate = ClientInfo.NodeTitleDate;
                        nowSam.SampleSource = ClientInfo.SampleSource;
                        nowSam.GroupingSituation = ClientInfo.GroupingSituation;
                        nowSam.Batch = ClientInfo.Batch;
                        nowSam.CaptureProbe = ClientInfo.CaptureProbe;
                        nowSam.Extracter = ClientInfo.Extracter;
                        nowSam.QualityPeople = ClientInfo.QualityPeople;
                        nowSam.LabOperator = ClientInfo.LabOperator;
                        nowSam.DeliveryDate = ClientInfo.DeliveryDate;
                        nowSam.Index_I5 = ClientInfo.Index_I5;
                        nowSam.Index_I7 = ClientInfo.Index_I7;
                        nowSam.SpotCheck = ClientInfo.SpotCheck;
                        nowSam.QualityManagement = ClientInfo.QualityManagement;
                        nowSam.ProcessMode = ClientInfo.ProcessMode;
                        nowSam.ClientDataSize = ClientInfo.ClientDataSize;
                        nowSam.DeplaneDataSize = ClientInfo.DeplaneDataSize;
                        nowSam.AdditionalNum = ClientInfo.AdditionalNum;
                        nowSam.FaultRemark = ClientInfo.FaultRemark;
                        nowSam.ClientUnit = ClientInfo.ClientUnit;
                        nowSam.Email = ClientInfo.Email;
                        nowSam.ProjectNumber = ClientInfo.ProjectNumber;
                        nowSam.DetectingPlatform = ClientInfo.DetectingPlatform;
                        InsertSysField(nowSam, user);
                        context.Insert(nowSam);
                    }
                    return true;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion


        #region 删除
        public bool DoDelClientInfo(List<Guid> requestModel)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    return context.Delete<ClientInfo>(a => requestModel.Contains(a.ClientInfoOID)) > 0;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion

        #region Excel导出
        public string DoExportClientInfo(LoginInfo user)
        {
            using (var context = new MsSqlContext(sqlConn))
            {
                Workbook workBook = new Workbook();
                Worksheet sheet = workBook.Worksheets[0];

                Directory.CreateDirectory(ConfigurationManager.AppSettings["ExportFilePath"].ToString());

                //设置EXCEL类型和基础信息及创建默认导出路径
                workBook.FileFormat = FileFormatType.Xlsx;
                TimeSpan ts = DateTime.UtcNow - new DateTime(1970, 1, 1, 0, 0, 0, 0);
                string dataTime = Convert.ToInt64(ts.TotalSeconds).ToString();
                string ExportTitle = "受检者信息";
                try
                {
                    List<ClientInfoDetailModel> thisQueryModel = context.JoinQuery<ClientInfo, Sys_Organize>((client, org) => new object[] {
                    JoinType.LeftJoin,client.SYS_ORG==org.Sys_OrganizeOID
                        })
                        //.Where((a, b) => b.ClassCode.StartsWith(user.OrgClassCode))
                        .Select((a, b) => a)
                        .Select(a => new ClientInfoDetailModel
                        {
                            ClientInfoOID = a.ClientInfoOID,
                            ItemsName = a.ItemsName,
                            ItemsType = a.ItemsType,
                            ExperimentalType = a.ExperimentalType,
                            Name = a.Name,
                            YKSampleNum = a.YKSampleNum,
                            RummageNum = a.RummageNum,
                            ClientSampleName = a.ClientSampleName,
                            Entity = a.Entity,
                            Sex = a.Sex,
                            ReceivedDate = a.ReceivedDate,
                            ProjectDate = a.ProjectDate,
                            NodeTitleDate = a.NodeTitleDate,
                            SampleType = a.SampleType,
                            SampleSource = a.SampleSource,
                            GroupingSituation = a.GroupingSituation,
                            Batch = a.Batch,
                            CaptureProbe = a.CaptureProbe,
                            Extracter = a.Extracter,
                            QualityPeople = a.QualityPeople,
                            LabOperator = a.LabOperator,
                            DeliveryDate = a.DeliveryDate,
                            Index_I5 = a.Index_I5,
                            Index_I7 = a.Index_I7,
                            SpotCheck = a.SpotCheck,
                            QualityManagement = a.QualityManagement,
                            ProcessMode = a.ProcessMode,
                            ClientDataSize = a.ClientDataSize,
                            DeplaneDataSize = a.DeplaneDataSize,
                            AdditionalNum = a.AdditionalNum,
                            FaultRemark = a.FaultRemark,
                            ClientUnit = a.ClientUnit,
                            Email = a.Email,
                            Phone = a.Phone,
                            ProjectNumber = a.ProjectNumber,
                            Birth = a.Birth,
                            IDCard = a.IDCard,
                            CompanyPhone = a.CompanyPhone,
                            SampleDetection = a.SampleDetection,
                            TechnologyDetection = a.TechnologyDetection,
                            DetectingPlatform = a.DetectingPlatform,
                            TNM = a.TNM,
                            IsEssential = a.IsEssential,
                            ClinicalDiagnosis = a.ClinicalDiagnosis,
                            FamilyHistory = a.FamilyHistory,
                            PersonalHistory = a.PersonalHistory
                        }).ToList();
                    List<string> RowFirst = new List<string>() {  "项目名称",
                        "项目类型",
                        "实验类型",
                        "客户姓名",
                        "YK样本编号",
                        "临检号",
                        "客户样本名称",
                        "癌种/病种",
                        "性别",
                        "收样日期",
                        "立项日期",
                        "预计结题日期",
                        "样本类型",
                        "样本来源",
                        "分组情况",
                        "批次",
                        "捕获探针",
                        "提取人",
                        "质检人",
                        "实验操作员",
                        "外送日期",
                        "index_I5",
                        "index_I7",
                        "文库自身抽检情况",
                        "公司质检情况",
                        "处理方式",
                        "客户数据量要求",
                        "实际下机数据量",
                        "加测、补做情况(次数)",
                        "操作失误备注",
                        "客户单位",
                        "联系邮箱",
                        "联系电话",
                        "立项编号",
                        "出生日期",
                        "身份证号码",
                        "联系电话",
                        "检测样本",       
                        "检测技术",     
                        "检测平台",     
                        "TNM分期 ",    
                        "是否原发",      
                        "临床诊断",    
                        "家族病史",    
                        "个人病史及用药史"
                        };
                    int cellNum = 0;
                    foreach (string item in RowFirst)
                    {
                        sheet.Cells[0, cellNum++].Value = item;
                    }
                    int rowNum = 1;

                    foreach (ClientInfoDetailModel a in thisQueryModel)
                    {
                        cellNum = 0;
                        sheet.Cells[rowNum, cellNum++].Value = a.ItemsName;
                        sheet.Cells[rowNum, cellNum++].Value = a.ItemsType;
                        sheet.Cells[rowNum, cellNum++].Value = a.ExperimentalType;
                        sheet.Cells[rowNum, cellNum++].Value = a.Name;
                        sheet.Cells[rowNum, cellNum++].Value = a.YKSampleNum;
                        sheet.Cells[rowNum, cellNum++].Value = a.RummageNum;
                        sheet.Cells[rowNum, cellNum++].Value = a.ClientSampleName;
                        sheet.Cells[rowNum, cellNum++].Value = a.Entity;
                        sheet.Cells[rowNum, cellNum++].Value = a.Sex==1?"男":"女";
                        sheet.Cells[rowNum, cellNum++].Value = a.ReceivedDate==null?"":((DateTime) a.ReceivedDate).ToString("yyyy-MM-dd");
                        sheet.Cells[rowNum, cellNum++].Value = a.ProjectDate == null?"":((DateTime)a.ProjectDate).ToString("yyyy-MM-dd");
                        sheet.Cells[rowNum, cellNum++].Value = a.NodeTitleDate == null ? "" : ((DateTime)a.NodeTitleDate).ToString("yyyy-MM-dd") ;       
                        sheet.Cells[rowNum, cellNum++].Value = a.SampleType;
                        sheet.Cells[rowNum, cellNum++].Value = a.SampleSource;
                        sheet.Cells[rowNum, cellNum++].Value = a.GroupingSituation;        
                        sheet.Cells[rowNum, cellNum++].Value = a.Batch;                    
                        sheet.Cells[rowNum, cellNum++].Value = a.CaptureProbe;             
                        sheet.Cells[rowNum, cellNum++].Value = a.Extracter;                
                        sheet.Cells[rowNum, cellNum++].Value = a.QualityPeople;            
                        sheet.Cells[rowNum, cellNum++].Value = a.LabOperator;              
                        sheet.Cells[rowNum, cellNum++].Value = a.DeliveryDate == null ? "":((DateTime)a.DeliveryDate).ToString("yyyy-MM-dd");             
                        sheet.Cells[rowNum, cellNum++].Value = a.Index_I5;                 
                        sheet.Cells[rowNum, cellNum++].Value = a.Index_I7;                 
                        sheet.Cells[rowNum, cellNum++].Value = a.SpotCheck;                
                        sheet.Cells[rowNum, cellNum++].Value = a.QualityManagement;        
                        sheet.Cells[rowNum, cellNum++].Value = a.ProcessMode;              
                        sheet.Cells[rowNum, cellNum++].Value = a.ClientDataSize;           
                        sheet.Cells[rowNum, cellNum++].Value = a.DeplaneDataSize;          
                        sheet.Cells[rowNum, cellNum++].Value = a.AdditionalNum;            
                        sheet.Cells[rowNum, cellNum++].Value = a.FaultRemark;              
                        sheet.Cells[rowNum, cellNum++].Value = a.ClientUnit;               
                        sheet.Cells[rowNum, cellNum++].Value = a.Email;                    
                        sheet.Cells[rowNum, cellNum++].Value = a.Phone;                    
                        sheet.Cells[rowNum, cellNum++].Value = a.ProjectNumber;            
                        sheet.Cells[rowNum, cellNum++].Value = a.Birth == null ? "" : ((DateTime)a.Birth).ToString("yyyy-MM-dd");                    
                        sheet.Cells[rowNum, cellNum++].Value = a.IDCard;
                        sheet.Cells[rowNum, cellNum++].Value = a.CompanyPhone;
                        sheet.Cells[rowNum, cellNum++].Value = a.SampleDetection;          
                        sheet.Cells[rowNum, cellNum++].Value = a.TechnologyDetection;      
                        sheet.Cells[rowNum, cellNum++].Value = a.DetectingPlatform;        
                        sheet.Cells[rowNum, cellNum++].Value = a.TNM;                      
                        sheet.Cells[rowNum, cellNum++].Value = a.IsEssential;              
                        sheet.Cells[rowNum, cellNum++].Value = a.ClinicalDiagnosis;        
                        sheet.Cells[rowNum, cellNum++].Value = a.FamilyHistory;
                        sheet.Cells[rowNum, cellNum++].Value = a.PersonalHistory;
                        rowNum++;
                    }

                    workBook.Save(ConfigurationManager.AppSettings["ExportFilePath"].ToString() + "\\" + ExportTitle + dataTime + ".xlsx");
                    return ConfigurationManager.AppSettings["MainWebSite"].ToString() + "/ExportFile/" + ExportTitle + dataTime + ".xlsx";

                }
                catch (Exception ex)
                {
                    throw new Exception(ex.Message);
                }
            }

        }

        /// <summary>
        /// 插入任务
        /// </summary>
        /// <param name="uer"></param>
        public Guid DoInsertMission(LoginInfo user)
        {
            Guid MissionOID = Guid.NewGuid();
            using (var context = new MsSqlContext(sqlConn))
            {
                Mission mission = new Mission()
                {
                    MissionOID = MissionOID,
                    MissionName = "导出受检者",
                    StartTime = DateTime.Now,

                    ExportPeople = user.Sys_UserOID,
                    CurrentProcess = "0%",
                    SYS_ORG = user.Sys_OrganizeOID
                };
                context.Insert(mission);
                return MissionOID;
            }
        }

        #endregion

        #region Excel导入
        public bool DoImportClientInfo(HttpPostedFile file, LoginInfo user)
        {

            IWorkbook workbook = null;
            ISheet sheet = null;
            try
            {
                if (file.FileName.Contains(".xlsx"))
                {
                    workbook = new XSSFWorkbook(file.InputStream);
                }
                else
                {
                    workbook = new HSSFWorkbook(file.InputStream);
                }
                sheet = workbook.GetSheetAt(0);
                int LastRowNum = sheet.LastRowNum;
                for (int i = 1; i <= LastRowNum; i++)
                {
                    IRow Irow = sheet.GetRow(i);
                    ClientInfoSaveRequestModel ClientInfo = new ClientInfoSaveRequestModel();
                    ClientInfo.ClientInfoOID = Guid.NewGuid();
                    ClientInfo.ItemsName = Irow.GetCell(0).ToString();
                    ClientInfo.ItemsType = Irow.GetCell(1).ToString();
                    ClientInfo.ExperimentalType = Irow.GetCell(2).ToString();
                    ClientInfo.Name = Irow.GetCell(3).ToString();

                    ClientInfo.YKSampleNum = Irow.GetCell(4).ToString();
                    ClientInfo.RummageNum = Irow.GetCell(5).ToString();
                    ClientInfo.ClientSampleName = Irow.GetCell(6).ToString();
                    ClientInfo.Entity = Irow.GetCell(7).ToString();
                    ClientInfo.Sex = Irow.GetCell(8) == null ? 1 : Irow.GetCell(8).ToString().Contains("男") ? 1 : 0;
                    if (Irow.GetCell(9) != null && !string.IsNullOrEmpty(Irow.GetCell(9).ToString()))
                    {
                        DateTime dtDate;
                        DateTime.TryParse(Irow.GetCell(9).ToString(), out dtDate);
                        ClientInfo.ReceivedDate = dtDate;
                    }

                    if (Irow.GetCell(10) != null &&!string.IsNullOrEmpty(Irow.GetCell(10).ToString()))
                    {
                        DateTime dtDate;
                        DateTime.TryParse(Irow.GetCell(10).ToString(), out dtDate);
                        ClientInfo.ProjectDate = dtDate;
                    }
                    if (Irow.GetCell(11) != null && !string.IsNullOrEmpty(Irow.GetCell(11).ToString()))
                    {
                        DateTime dtDate;
                        DateTime.TryParse(Irow.GetCell(11).ToString(), out dtDate);
                        ClientInfo.NodeTitleDate = dtDate;
                    }
                    ClientInfo.SampleType = Irow.GetCell(12).ToString();
                    ClientInfo.SampleSource = Irow.GetCell(13).ToString();
                    ClientInfo.GroupingSituation = Irow.GetCell(14).ToString();
                    ClientInfo.Batch = Irow.GetCell(15).ToString();
                    ClientInfo.CaptureProbe = Irow.GetCell(16).ToString();
                    ClientInfo.Extracter = Irow.GetCell(17).ToString();
                    ClientInfo.QualityPeople = Irow.GetCell(18).ToString();
                    ClientInfo.LabOperator = Irow.GetCell(19).ToString();
                    if (Irow.GetCell(20) != null && !string.IsNullOrEmpty(Irow.GetCell(20).ToString()))
                    {

                        DateTime dtDate;
                        DateTime.TryParse(Irow.GetCell(20).ToString(), out dtDate);
                        ClientInfo.DeliveryDate = dtDate;
                    }
                    ClientInfo.Index_I5 = Irow.GetCell(21).ToString();
                    ClientInfo.Index_I7 = Irow.GetCell(22).ToString();

                    ClientInfo.SpotCheck = Irow.GetCell(23).ToString();

                    ClientInfo.QualityManagement = Irow.GetCell(24).ToString();
                    ClientInfo.ProcessMode = Irow.GetCell(25).ToString();
                    ClientInfo.ClientDataSize = Irow.GetCell(26).ToString();
                    ClientInfo.DeplaneDataSize = Irow.GetCell(27).ToString();
                    ClientInfo.AdditionalNum = Irow.GetCell(28).ToString();
                    ClientInfo.FaultRemark = Irow.GetCell(29).ToString();
                    ClientInfo.ClientUnit = Irow.GetCell(30).ToString();
                    ClientInfo.Email = Irow.GetCell(31).ToString();
                    ClientInfo.Phone = Irow.GetCell(32).ToString();
                    ClientInfo.ProjectNumber = Irow.GetCell(33).ToString();
                    if (Irow.GetCell(34) != null && !string.IsNullOrEmpty(Irow.GetCell(34).ToString()))
                    {
                        DateTime dtDate;
                        DateTime.TryParse(Irow.GetCell(34).ToString(), out dtDate);
                        ClientInfo.Birth = dtDate;
                    }
                    ClientInfo.IDCard = Irow.GetCell(35).ToString();
                    ClientInfo.CompanyPhone = Irow.GetCell(36).ToString();
                    ClientInfo.SampleDetection = Irow.GetCell(37).ToString();
                    ClientInfo.TechnologyDetection = Irow.GetCell(38).ToString();
                    ClientInfo.DetectingPlatform = Irow.GetCell(39).ToString();
                    ClientInfo.TNM = Irow.GetCell(40).ToString();
                    ClientInfo.IsEssential = Irow.GetCell(41) == null ? true : Irow.GetCell(40).ToString().Contains("是") ? true : false;
                    ClientInfo.ClinicalDiagnosis = Irow.GetCell(42).ToString();
                    ClientInfo.FamilyHistory = Irow.GetCell(43).ToString();
                    ClientInfo.PersonalHistory = Irow.GetCell(44).ToString();


                    DoSaveClientInfo(ClientInfo, user);
                }

                return true;
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion
    }
}
