﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Chloe.SqlServer;
using Common.Unit;
using G.Buss.BaseInfo.ViewModel.CodeTableModel;
using G.MES.DataModel;
using G.MES.ViewModel;
using Microsoft.SqlServer.Server;

namespace G.Buss.BaseInfo.Services.CodeTableSVC
{

    public class CodeTableOperaSVC : BaseSvc
    {
        /// <summary>
        /// 获取全部数据字典
        /// </summary>
        /// <param name="requestModel"></param>
        /// <returns></returns>
        public List<AllCodeTableResponseModel> DoGetAllCodeTable(AllCodeTableRequestModel requestModel)
        {
            try
            {
                var where = new LamadaExtention<AllCodeTableResponseModel>();
                if (requestModel != null)
                {
                    if (!string.IsNullOrEmpty(requestModel.QueryValue))
                    {
                        where.And(a => a.CodeTableID.Contains(requestModel.QueryValue) || a.CodeTableName.Contains(requestModel.QueryValue));
                    }
                }

                using (var context = new MsSqlContext(sqlConn))
                {
                    List<AllCodeTableResponseModel> responseModels = context.Query<CodeClassify>()
                        .Select(a => new AllCodeTableResponseModel
                        {
                            CodeClassifyOID = a.CodeClassifyOID,
                            CodeTableID = a.CodeTableID,
                            CodeTableName = a.CodeTableName
                        }).Where(where.GetLambda()).ToList();
                    return responseModels;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        /// <summary>
        /// 删除数据字典带判断
        /// </summary>
        /// <param name="Initi06OID"></param>
        /// <returns></returns>
        public bool DoDelCodeTable(Guid CodeClassifyOID)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    //判断是否有子级
                    int SonExistNum = context.Query<CodeValue>().Where(a => a.CodeClassifyOID == CodeClassifyOID).ToList().Count();
                    if (SonExistNum > 0)
                    {
                        throw new Exception("该数据字典存在明细，不可删除");
                    }
                    context.Delete<CodeClassify>(a => a.CodeClassifyOID == CodeClassifyOID);
                    return true;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        /// <summary>
        /// 新增或更新
        /// </summary>
        /// <param name="requestModel"></param>
        /// <returns></returns>
        public bool DoSaveORUpdateCodeTable(SaveORUpdateCodeTableRequestModel requestModel)
        {
            try
            {
                
                using (var context = new MsSqlContext(sqlConn))
                {
                    
                        CodeClassify CodeClassify = new CodeClassify();
                        CodeClassify.CodeClassifyOID = requestModel.CodeClassifyOID;
                        context.TrackEntity(CodeClassify);
                        CodeClassify.CodeTableID = requestModel.CodeTableID;
                        CodeClassify.CodeTableName = requestModel.CodeTableName;
                        IsTableIDExist(requestModel.CodeTableID, requestModel.CodeClassifyOID);

                        int SonExistNum = context.Query<CodeClassify>().Where(a => a.CodeClassifyOID == requestModel.CodeClassifyOID).ToList().Count();
                        if (SonExistNum > 0)
                        {
                            //更新
                            context.Update<CodeClassify>(CodeClassify);
                        }
                        else
                        {
                            //新增
                            context.Insert<CodeClassify>(CodeClassify);

                        }
                    return true;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        /// <summary>
        /// 判断数据编码是否重复
        /// </summary>
        /// <param name="CodeTableID"></param>
        public void IsTableIDExist(string CodeTableID, Guid OID)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    int ExistNum = context.Query<CodeClassify>().Where(a => a.CodeTableID == CodeTableID&&a.CodeClassifyOID != OID).ToList().Count();
                    if (ExistNum > 0)
                    {
                        throw new Exception("该数据字典ID编码已存在");
                    }
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        /// <summary>
        /// 获取明细列表，分页
        /// </summary>
        /// <param name="requestModel"></param>
        /// <returns></returns>
        public PageData DoGetSonCodeTable(SonCodeTableRequestModel requestModel)
        {
            try
            {
                var where = new LamadaExtention<SonCodeTableResponseModel>();
                if (!string.IsNullOrEmpty(requestModel.CodeID))
                {
                    where.And(a => a.CodeID.Contains(requestModel.CodeID));
                }
                if (!string.IsNullOrEmpty(requestModel.CodeName))
                {
                    where.And(a => a.CodeName.Contains(requestModel.CodeName));
                }

                using (var context = new MsSqlContext(sqlConn))
                {
                    var AllSonDictionaryDirectory = context.Query<CodeValue>()
                        .Where(a => a.CodeClassifyOID == requestModel.CodeClassifyOID)
                        .Select(a => new SonCodeTableResponseModel
                        {
                            CodeValueOID = a.CodeValueOID,
                            CodeID = a.CodeID,
                            CodeName = a.CodeName,
                            OrderNo = a.OrderNo,
                            CodeTableID = a.CodeTableID,
                            Reserved3 = a.Reserved3
                        }).Where(where.GetLambda()).OrderBy(a=>a.OrderNo);
                    
                    //.Where(where.GetLambda())
                    return PageData.Make(
                        requestModel.PageNumber,
                        requestModel.PageSize,
                        AllSonDictionaryDirectory.Count(),
                        AllSonDictionaryDirectory.TakePage(requestModel.PageNumber, requestModel.PageSize).ToList());
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        /// <summary>
        /// 获取明细列表，全部
        /// </summary>
        /// <param name="requestModel"></param>
        /// <returns></returns>
        public List<SonCodeTableResponseModel> DoGetAllSonCodeTable(string CodeTableID)
        {
            try
            {

                using (var context = new MsSqlContext(sqlConn))
                {
                    List<SonCodeTableResponseModel> AllSonDictionaryDirectory = context.Query<CodeValue>()
                        .Where(a => a.CodeTableID == CodeTableID)
                        .Select(a => new SonCodeTableResponseModel
                        {
                            CodeID = a.CodeID,
                            CodeName = a.CodeName,
                            OrderNo = a.OrderNo,
                           Reserved3 = a.Reserved3
                }).ToList();

                    return AllSonDictionaryDirectory;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        /// <summary>
        /// 更新或修改
        /// </summary>
        /// <returns></returns>
        public bool DoSaveORUpdateSonCodeTable(SaveORUpdateSonCodeTableListRequestModel requestModelList)
        {

            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    foreach (SaveORUpdateSonCodeTableRequestModel requestModel in requestModelList.CodeValueList)
                    {
                        CodeValue NowCodeValue = new CodeValue();
                        NowCodeValue.CodeValueOID = requestModel.CodeValueOID;
                        NowCodeValue.CodeClassifyOID = requestModel.CodeClassifyOID;
                        context.TrackEntity(NowCodeValue);
                        NowCodeValue.CodeID = requestModel.CodeID;
                        NowCodeValue.CodeTableID = requestModel.CodeTableID;
                        NowCodeValue.CodeName = requestModel.CodeName;
                        NowCodeValue.CodeTableID = requestModel.CodeTableID;
                        NowCodeValue.OrderNo = requestModel.OrderNo;
                        NowCodeValue.Reserved3 = requestModel.Reserved3;
                        NowCodeValue.ParentOID = Guid.Empty;
                        int SonExistNum = context.Query<CodeValue>().Where(a => a.CodeValueOID == requestModel.CodeValueOID).ToList().Count();
                        if (SonExistNum > 0)
                        {
                            //更新
                            context.Update<CodeValue>(NowCodeValue);
                        }
                        else
                        {
                            //新增
                            context.Insert<CodeValue>(NowCodeValue);

                        }
                    }
                  
                    return true;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        public bool DoDelSonCodeTable(List<Guid> CodeValueOIDList)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    context.Delete<CodeValue>(a => CodeValueOIDList.Contains(a.CodeValueOID));
                }
                return true;
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
    }
}
