﻿using Chloe;
using Chloe.SqlServer;
using Common.Unit;
using G.Buss.BaseInfo.Services.CodeTableSVC;
using G.Buss.BaseInfo.ViewModel.CodeTableModel;
using G.Buss.BaseInfo.ViewModel.GeneCorrelationModel;
using G.MES.DataModel;
using G.MES.ViewModel;
using NPOI.HSSF.UserModel;
using NPOI.SS.UserModel;
using NPOI.XSSF.UserModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;

namespace G.Buss.BaseInfo.Services.GeneCorrelationSVC
{
    public  class GeneCorrelationSVC:BaseSvc
    {
        #region 获取列表（带查）
        public PageData DoGetGeneCorrelation(GeneCorrelationRequestModel requestModel, LoginInfo user)
        {
            try
            {
                GetWhere(requestModel, user, out LamadaExtention<GeneCorrelationQueryModel> where);
                using (var context = new MsSqlContext(sqlConn))
                {
                    var GeneCorrelationQuery = context.JoinQuery<GeneCorrelation, Sys_Organize>((GeneCorrelation, org) => new object[] {
                        JoinType.LeftJoin,GeneCorrelation.SYS_ORG == org.Sys_OrganizeOID
                    }).Select((GeneCorrelation, org) => new GeneCorrelationQueryModel
                    {
                        GeneCorrelationOID = GeneCorrelation.GeneCorrelationOID,
                        GeneName = GeneCorrelation.GeneName,
                        SortNo = GeneCorrelation.SortNo,
                        Type = GeneCorrelation.Type+"",
                        ClassCode = org.ClassCode
                    }).Where(where.GetLambda());

                    int TotalNum = GeneCorrelationQuery.Count();

                    List<GeneCorrelationQueryModel> GeneCorrelationList = GeneCorrelationQuery.OrderBy(a => a.SortNo).TakePage(requestModel.pageindex, requestModel.pagesize).ToList();

                    return PageData.Make(requestModel.pageindex, requestModel.pagesize, TotalNum, GeneCorrelationList);
                }

            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        /// <summary>
        /// 获取筛选条件
        /// </summary>
        /// <param name="requestModel"></param>
        /// <param name="user"></param>
        /// <returns></returns>
        public void GetWhere(GeneCorrelationRequestModel requestModel, LoginInfo user, out LamadaExtention<GeneCorrelationQueryModel> where)
        {
            try
            {
                where = new LamadaExtention<GeneCorrelationQueryModel>();
                if (!string.IsNullOrEmpty(requestModel.QueryCondition))
                {
                    where.And(a =>a.GeneName.Contains(requestModel.QueryCondition));
                }
                if (user.Sys_UserOID != new Guid() && user.Sys_UserOID != new Guid("00000000-0000-0000-0000-000000000001"))
                {
                    where.And(a => a.ClassCode.StartsWith(user.OrgClassCode));
                }
            }
            catch (Exception ex)
            {
                throw new Exception("获取筛选条件时：" + ex.Message);
            }

        }
        #endregion


        #region 保存（带更新）
        public bool DoSaveGeneCorrelation(GeneCorrelationSaveRequestModel GeneCorrelation, LoginInfo user)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    GeneCorrelation nowSam = context.Query<GeneCorrelation>().Where(a => a.GeneName == GeneCorrelation.GeneName).FirstOrDefault();
                    if (nowSam != null)
                    {
                        //更新
                        context.TrackEntity(nowSam);
                        nowSam.GeneName = GeneCorrelation.GeneName;
                        nowSam.SortNo = GeneCorrelation.SortNo;
                        nowSam.Type = GeneCorrelation.Type;
                        UpSysField(nowSam, user);
                        context.Update(nowSam);
                    }
                    else
                    {
                        //新增
                        nowSam = new GeneCorrelation();
                        nowSam.GeneCorrelationOID = GeneCorrelation.GeneCorrelationOID;
                        nowSam.GeneName = GeneCorrelation.GeneName;
                        nowSam.SortNo = GeneCorrelation.SortNo;
                        nowSam.Type = GeneCorrelation.Type;
                        InsertSysField(nowSam, user);
                        context.Insert(nowSam);
                    }
                    return true;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion


        #region 删除
        public bool DoDelGeneCorrelation(List<Guid> requestModel)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    return context.Delete<GeneCorrelation>(a => requestModel.Contains(a.GeneCorrelationOID)) > 0;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion

        #region Excel导入
        public bool DoImportGeneCorrelation(HttpPostedFile file, LoginInfo user)
        {

            IWorkbook workbook = null;
            ISheet sheet = null;
            CodeTableOperaSVC code = new CodeTableOperaSVC();
            List<SonCodeTableResponseModel> SonCodeTable = code.DoGetAllSonCodeTable("Code_GeneType");
            Dictionary<string, string> CodeIDAndName = new Dictionary<string, string>();
            foreach (SonCodeTableResponseModel item in SonCodeTable)
            {
                CodeIDAndName.Add(item.CodeName,item.CodeID);
            }
            try
            {
                if (file.FileName.Contains(".xlsx"))
                {
                    workbook = new XSSFWorkbook(file.InputStream);
                }
                else
                {
                    workbook = new HSSFWorkbook(file.InputStream);
                }
                sheet = workbook.GetSheetAt(0);
                int LastRowNum = sheet.LastRowNum;
           

                for (int i = 1; i <= LastRowNum; i++)
                {
                    IRow Irow = sheet.GetRow(i);
                    GeneCorrelationSaveRequestModel GeneCorrelation = new GeneCorrelationSaveRequestModel();
                    GeneCorrelation.GeneCorrelationOID = Guid.NewGuid();
                    GeneCorrelation.GeneName = Irow.GetCell(0).ToString();
                    GeneCorrelation.SortNo = Convert.ToInt32( Irow.GetCell(1).ToString());
                    string typeMess = Irow.GetCell(2).ToString();
                    if (CodeIDAndName.ContainsKey(typeMess))
                    {
                        GeneCorrelation.Type = CodeIDAndName[typeMess];
                    }
                    DoSaveGeneCorrelation(GeneCorrelation, user);
                }

                return true;
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion
    }
}
