﻿using Chloe;
using Chloe.SqlServer;
using Common.Unit;
using G.Buss.BaseInfo.ViewModel.GeneticRisksModel;
using G.MES.DataModel;
using G.MES.ViewModel;
using NPOI.HSSF.UserModel;
using NPOI.SS.UserModel;
using NPOI.XSSF.UserModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;

namespace G.Buss.BaseInfo.Services.GeneticRisksSVC
{
    public class GeneticRisksSVC:BaseSvc
    {
        #region 获取列表（带查）
        public PageData DoGetGeneticRisks(GeneticRisksRequestModel requestModel, LoginInfo user)
        {
            try
            {
                GetWhere(requestModel, user, out LamadaExtention<GeneticRisksQueryModel> where);
                using (var context = new MsSqlContext(sqlConn))
                {
                    var GeneticRisksQuery = context.JoinQuery<GeneticRisks, Sys_Organize>((GeneticRisks, org) => new object[] {
                        JoinType.LeftJoin,GeneticRisks.SYS_ORG == org.Sys_OrganizeOID
                    }).Select((GeneticRisks, org) => new GeneticRisksQueryModel
                    {
                        GeneticRisksOID = GeneticRisks.GeneticRisksOID,
                        SampleCode = GeneticRisks.SampleCode,
                        GeneName = GeneticRisks.GeneName,
                        Mutations = GeneticRisks.Mutations,
                        MutationType = GeneticRisks.MutationType,
                        Heterozygosity =  GeneticRisks.Heterozygosity,
                        RiskAssessment =  GeneticRisks.RiskAssessment,
                        ClassCode = org.ClassCode
                    }).Where(where.GetLambda());

                    int TotalNum = GeneticRisksQuery.Count();

                    List<GeneticRisksQueryModel> GeneticRisksList = GeneticRisksQuery.OrderBy(a => a.SampleCode).TakePage(requestModel.pageindex, requestModel.pagesize).ToList();

                    return PageData.Make(requestModel.pageindex, requestModel.pagesize, TotalNum, GeneticRisksList);
                }

            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        /// <summary>
        /// 获取筛选条件
        /// </summary>
        /// <param name="requestModel"></param>
        /// <param name="user"></param>
        /// <returns></returns>
        public void GetWhere(GeneticRisksRequestModel requestModel, LoginInfo user, out LamadaExtention<GeneticRisksQueryModel> where)
        {
            try
            {
                where = new LamadaExtention<GeneticRisksQueryModel>();
                if (!string.IsNullOrEmpty(requestModel.QueryCondition))
                {
                    where.And(a => a.SampleCode.Contains(requestModel.QueryCondition)
                    || a.GeneName.Contains(requestModel.QueryCondition)
                    || a.Mutations.Contains(requestModel.QueryCondition)
                    || a.MutationType.Contains(requestModel.QueryCondition)
                    || a.Heterozygosity.Contains(requestModel.QueryCondition)
                    || a.RiskAssessment.Contains(requestModel.QueryCondition)
                    );
                }


                if (user.Sys_UserOID != new Guid() && user.Sys_UserOID != new Guid("00000000-0000-0000-0000-000000000001"))
                {
                    where.And(a => a.ClassCode.StartsWith(user.OrgClassCode));
                }
            }
            catch (Exception ex)
            {
                throw new Exception("获取筛选条件时：" + ex.Message);
            }

        }
        #endregion


        #region 保存（带更新）
        public bool DoSaveGeneticRisks(GeneticRisksSaveRequestModel GeneticRisks, LoginInfo user)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    GeneticRisks nowSam = context.Query<GeneticRisks>().Where(a => a.SampleCode == GeneticRisks.SampleCode).FirstOrDefault();
                    if (nowSam != null)
                    {
                        //更新
                        context.TrackEntity(nowSam);
                        nowSam.GeneName = GeneticRisks.GeneName;
                        nowSam.SampleCode = GeneticRisks.SampleCode;
                        nowSam.Mutations = GeneticRisks.Mutations;
                        nowSam.MutationType = GeneticRisks.MutationType;
                        nowSam.Heterozygosity = GeneticRisks.Heterozygosity;
                        nowSam.RiskAssessment = GeneticRisks.RiskAssessment;
                        UpSysField(nowSam, user);
                        context.Update(nowSam);
                    }
                    else
                    {
                        //新增
                        nowSam = new GeneticRisks();
                        nowSam.GeneticRisksOID = GeneticRisks.GeneticRisksOID;
                        nowSam.GeneName = GeneticRisks.GeneName;
                        nowSam.SampleCode = GeneticRisks.SampleCode;
                        nowSam.Mutations = GeneticRisks.Mutations;
                        nowSam.MutationType = GeneticRisks.MutationType;
                        nowSam.Heterozygosity = GeneticRisks.Heterozygosity;
                        nowSam.RiskAssessment = GeneticRisks.RiskAssessment;
                        InsertSysField(nowSam, user);
                        context.Insert(nowSam);
                    }
                    return true;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion


        #region 删除
        public bool DoDelGeneticRisks(List<Guid> requestModel)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    return context.Delete<GeneticRisks>(a => requestModel.Contains(a.GeneticRisksOID)) > 0;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion

        #region Excel导入
        public bool DoImportGeneticRisks(HttpPostedFile file, LoginInfo user)
        {

            IWorkbook workbook = null;
            ISheet sheet = null;
            using (var context = new MsSqlContext(sqlConn)) {
                context.Session.BeginTransaction();
                try
                {
                    if (file.FileName.Contains(".xlsx"))
                    {
                        workbook = new XSSFWorkbook(file.InputStream);
                    }
                    else
                    {
                        workbook = new HSSFWorkbook(file.InputStream);
                    }
                    sheet = workbook.GetSheetAt(0);
                    int LastRowNum = sheet.LastRowNum;
                    for (int i = 1; i <= LastRowNum; i++)
                    {
                        IRow Irow = sheet.GetRow(i);
                        GeneticRisks GeneticRisks = new GeneticRisks();
                        GeneticRisks.GeneticRisksOID = Guid.NewGuid();
                        GeneticRisks.SampleCode = Irow.GetCell(0).ToString();
                        GeneticRisks.GeneName = Irow.GetCell(1).ToString();
                        GeneticRisks.Mutations = Irow.GetCell(2).ToString();
                        GeneticRisks.MutationType = Irow.GetCell(3).ToString();
                        GeneticRisks.Heterozygosity = Irow.GetCell(4).ToString();
                        GeneticRisks.RiskAssessment = Irow.GetCell(5).ToString();
                        InsertSysField(GeneticRisks, user);
                        context.Insert(GeneticRisks);
                    }
                   
                    context.Session.CommitTransaction();
                    return true;
                }
                catch (Exception ex)
                {
                    context.Session.RollbackTransaction();
                    throw new Exception(ex.Message);
                }

            }
                
        }
        #endregion
    }
}
