﻿using Chloe;
using Chloe.SqlServer;
using Common.Unit;
using G.Buss.BaseInfo.ViewModel.SampleControlModel;
using G.MES.DataModel;
using G.MES.ViewModel;
using NPOI.HSSF.UserModel;
using NPOI.SS.UserModel;
using NPOI.XSSF.UserModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;

namespace G.Buss.BaseInfo.Services
{
    public class SampleControlSVC:BaseSvc
    {
        #region 获取列表（带查）
        public PageData DoGetSampleControl(SampleControlRequestModel requestModel, LoginInfo user)
        {
            try
            {
                GetWhere(requestModel, user, out LamadaExtention<SampleControlQueryModel> where);
                using (var context = new MsSqlContext(sqlConn))
                {
                    var SampleControlQuery = context.JoinQuery<SampleControl, Sys_Organize>((SampleControl, org) => new object[] {
                        JoinType.LeftJoin,SampleControl.SYS_ORG == org.Sys_OrganizeOID
                    }).Select((SampleControl, org) => new SampleControlQueryModel
                    {
                        SampleControlOID = SampleControl.SampleControlOID,
                        ControlStep = SampleControl.ControlStep,
                        ControlResult = SampleControl.ControlResult,
                        SortNo = SampleControl.SortNo,
                        ClassCode = org.ClassCode
                    }).Where(where.GetLambda());

                    int TotalNum = SampleControlQuery.Count();

                    List<SampleControlQueryModel> SampleControlList = SampleControlQuery.OrderBy(a => a.SortNo).TakePage(requestModel.pageindex, requestModel.pagesize).ToList();

                    return PageData.Make(requestModel.pageindex, requestModel.pagesize, TotalNum, SampleControlList);
                }

            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        /// <summary>
        /// 获取筛选条件
        /// </summary>
        /// <param name="requestModel"></param>
        /// <param name="user"></param>
        /// <returns></returns>
        public void GetWhere(SampleControlRequestModel requestModel, LoginInfo user, out LamadaExtention<SampleControlQueryModel> where)
        {
            try
            {
                where = new LamadaExtention<SampleControlQueryModel>();
                if (!string.IsNullOrEmpty(requestModel.QueryCondition))
                {
                    where.And(a => a.ControlStep.Contains(requestModel.QueryCondition)
                    || a.ControlResult.Contains(requestModel.QueryCondition)
                    );
                }
            
              
                if (user.Sys_UserOID != new Guid() && user.Sys_UserOID != new Guid("00000000-0000-0000-0000-000000000001"))
                {
                    where.And(a => a.ClassCode.StartsWith(user.OrgClassCode));
                }
            }
            catch (Exception ex)
            {
                throw new Exception("获取筛选条件时：" + ex.Message);
            }

        }
        #endregion


        #region 保存（带更新）
        public bool DoSaveSampleControl(SampleControlSaveRequestModel SampleControl, LoginInfo user)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    SampleControl nowSam = context.Query<SampleControl>().Where(a => a.SampleControlOID == SampleControl.SampleControlOID).FirstOrDefault();
                    if (nowSam != null)
                    {
                        //更新
                        context.TrackEntity(nowSam);
                        nowSam.ControlResult = SampleControl.ControlResult;
                        nowSam.ControlStep = SampleControl.ControlStep;
                        nowSam.SortNo = SampleControl.SortNo;
                        UpSysField(nowSam, user);
                        context.Update(nowSam);
                    }
                    else
                    {
                        //新增
                        nowSam = new SampleControl();
                        nowSam.SampleControlOID = SampleControl.SampleControlOID;
                        nowSam.ControlResult = SampleControl.ControlResult;
                        nowSam.ControlStep = SampleControl.ControlStep;
                        nowSam.SortNo = SampleControl.SortNo;
                        InsertSysField(nowSam, user);
                        context.Insert(nowSam);
                    }
                    return true;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion


        #region 删除
        public bool DoDelSampleControl(List<Guid> requestModel)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    return context.Delete<SampleControl>(a => requestModel.Contains(a.SampleControlOID)) > 0;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion

        #region Excel导入
        public bool DoImportSampleControl(HttpPostedFile file, LoginInfo user)
        {

            IWorkbook workbook = null;
            ISheet sheet = null;
            try
            {
                if (file.FileName.Contains(".xlsx"))
                {
                    workbook = new XSSFWorkbook(file.InputStream);
                }
                else
                {
                    workbook = new HSSFWorkbook(file.InputStream);
                }
                sheet = workbook.GetSheetAt(0);
                int LastRowNum = sheet.LastRowNum;
                for (int i = 1; i <= LastRowNum; i++)
                {
                    IRow Irow = sheet.GetRow(i);
                    SampleControlSaveRequestModel SampleControl = new SampleControlSaveRequestModel();
                    SampleControl.SampleControlOID = Guid.NewGuid();
                    SampleControl.ControlStep = Irow.GetCell(0).ToString();
                    SampleControl.ControlResult = Irow.GetCell(1).ToString();
                    SampleControl.SortNo = Convert.ToInt32( Irow.GetCell(2).ToString());
                    DoSaveSampleControl(SampleControl, user);
                }

                return true;
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion
    }
}
