﻿using Chloe;
using Chloe.SqlServer;
using Common.Unit;
using G.Buss.BaseInfo.ViewModel.SampleTypeModel;
using G.MES.DataModel;
using G.MES.ViewModel;
using NPOI.HSSF.UserModel;
using NPOI.SS.UserModel;
using NPOI.XSSF.UserModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Web;

namespace G.Buss.BaseInfo.Services.SampleTypeSVC
{
    public class SampleTypeSVC : BaseSvc
    {
        #region 获取列表（带查）
        public PageData DoGetSampleType(SampleTypeRequestModel requestModel, LoginInfo user)
        {
            try
            {
                GetWhere(requestModel, user, out LamadaExtention<SampleTypeQueryModel> where);
                using (var context = new MsSqlContext(sqlConn))
                {
                    var sampleTypeQuery = context.JoinQuery<SampleType,ClientInfo, Sys_Organize>((sample,client, org) => new object[] {
                        JoinType.LeftJoin,sample.SYS_ORG == org.Sys_OrganizeOID,
                        JoinType.LeftJoin,sample.SampleCode == client.SampleCode
                    }).Select((sample, client,org) => new SampleTypeQueryModel
                    {
                        SampleTypeOID = sample.SampleTypeOID,
                        SampleCode = sample.SampleCode,
                        SampleName = sample.SampleName,
                        ClassCode = org.ClassCode,
                        ClientOID = client.ClientInfoOID
                    }).Where(where.GetLambda());

                    int TotalNum = sampleTypeQuery.Count();

                    List<SampleTypeQueryModel> sampleTypeList = sampleTypeQuery.OrderBy(a=>a.SampleCode).TakePage(requestModel.pageindex, requestModel.pagesize).ToList();

                    return PageData.Make(requestModel.pageindex, requestModel.pagesize, TotalNum, sampleTypeList);
                }

            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        public List<SampleTypeSelectModel> DoGetAllSampleType( LoginInfo user)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    var sampleTypeQuery = context.JoinQuery<SampleType, Sys_Organize>((sample, org) => new object[] {
                        JoinType.LeftJoin,sample.SYS_ORG == org.Sys_OrganizeOID
                    });
                    if(user.Sys_UserOID!=new Guid("00000000-0000-0000-0000-000000000001"))
                    {
                        sampleTypeQuery = sampleTypeQuery.Where((sample, org) => org.ClassCode.StartsWith(user.OrgClassCode));
                    }
                    List<SampleTypeSelectModel> sampleTypeAll = sampleTypeQuery.Select((sample, org) => new SampleTypeSelectModel
                        {
                            SampleCode = sample.SampleCode,
                            SampleName = sample.SampleName,
                        }).ToList();
                    
                    return sampleTypeAll;
                }

            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        


        /// <summary>
        /// 获取筛选条件
        /// </summary>
        /// <param name="requestModel"></param>
        /// <param name="user"></param>
        /// <returns></returns>
        public void GetWhere(SampleTypeRequestModel requestModel, LoginInfo user, out LamadaExtention<SampleTypeQueryModel> where)
        {
            try
            {
                where = new LamadaExtention<SampleTypeQueryModel>();
                if (!string.IsNullOrEmpty(requestModel.QueryCondition))
                {
                    where.And(a => a.SampleCode.Contains(requestModel.QueryCondition) || a.SampleName.Contains(requestModel.QueryCondition));
                }
             
                if (user.Sys_UserOID != new Guid() && user.Sys_UserOID != new Guid("00000000-0000-0000-0000-000000000001"))
                {
                    where.And(a => a.ClassCode.StartsWith(user.OrgClassCode));
                }
            }
            catch (Exception ex)
            {
                throw new Exception("获取筛选条件时：" + ex.Message);
            }

        }
        #endregion


        #region 保存（带更新）
        public bool DoSaveSampleType(SampleTypeSaveRequestModel Sample, LoginInfo user)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    SampleType nowSam = context.Query<SampleType>().Where(a => a.SampleCode == Sample.SampleCode).FirstOrDefault();
                    if (nowSam != null)
                    {
                        //更新
                        context.TrackEntity(nowSam);
                        nowSam.SampleName = Sample.SampleName;
                        nowSam.SampleCode = Sample.SampleCode;
                        UpSysField(nowSam, user);
                        context.Update(nowSam);
                    }
                    else
                    {
                        //新增
                        nowSam = new SampleType();
                        nowSam.SampleTypeOID = Sample.SampleTypeOID;
                        nowSam.SampleName = Sample.SampleName;
                        nowSam.SampleCode = Sample.SampleCode;
                        InsertSysField(nowSam, user);
                        context.Insert(nowSam);
                    }
                    return true;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion

        #region 删除
        public bool DoDelSampleType(List<Guid> requestModel)
        {
            try
            {
                using (var context = new MsSqlContext(sqlConn))
                {
                    return context.Delete<SampleType>(a => requestModel.Contains(a.SampleTypeOID)) > 0;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion

        #region Excel导入
        public bool DoImportSampleType(HttpPostedFile file,LoginInfo user)
        {
            
            IWorkbook workbook = null;
            ISheet sheet = null;
            try
            {
                if (file.FileName.Contains(".xlsx"))
                {
                    workbook = new XSSFWorkbook(file.InputStream);
                }
                else
                {
                    workbook = new HSSFWorkbook(file.InputStream);
                }
                sheet = workbook.GetSheetAt(0);
                int LastRowNum = sheet.LastRowNum;
                for (int i = 1; i <= LastRowNum; i++)
                {
                    IRow Irow = sheet.GetRow(i);
                    SampleTypeSaveRequestModel sample = new SampleTypeSaveRequestModel();
                    sample.SampleTypeOID = Guid.NewGuid();
                    sample.SampleCode = Irow.GetCell(0).ToString();
                    sample.SampleName = Irow.GetCell(1).ToString();
                    DoSaveSampleType(sample, user);
                }

                return true;
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }
        #endregion


    }
}
