﻿using Chloe;
using Common.Unit;
using G.Buss.Client.ViewModel;
using G.MES.DataModel;
using G.MES.ViewModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace G.Buss.Client
{
    /// <summary>终端管理服务</summary>
    public class ClientMgrSvc : BaseSvc
    {

        /// <summary>
        /// 注册终端
        /// </summary>
        /// <param name="one">终端的信息</param>
        /// <param name="loginerOID">登录人FK</param>
        /// <param name="orgOID">组织机构FK</param>
        /// <returns></returns>
        public string AddClient(ClientMgrVM one, Guid loginerOID, Guid orgOID)
        {
            Guid key = Guid.NewGuid();
            int existCount = context.Query<Client_Mgr>().Count();
            string id = existCount.ToString().PadLeft(5, '0');
            Client_Mgr item = new Client_Mgr();
            item.Client_MgrOID = key;
            item.ID = id;
            item.Name = one.Name;
            item.IP = one.IP;
            item.GYLXFK = Guid.Parse(one.GYLXFK);
            item.Note = one.Note;
            item.Online = false;
            item.PCConfig = "";
            item.NotifyKey = "";
            item.SYS_Created = DateTime.Now;
            item.SYS_LAST_UPD = DateTime.Now;
            item.SYS_CreatedBy = loginerOID;
            item.SYS_ORG = orgOID;
            item.SYS_LAST_UPD_BY = loginerOID;
            context.Insert<Client_Mgr>(item);
            return key.ToString();
        }

        /// <summary>
        /// 修改终端的信息
        /// </summary>
        /// <param name="one">修改的信息</param>
        /// <param name="loginerOID">登录人FK</param>
        /// <param name="orgOID">登录人FK</param>
        /// <returns></returns>
        public string UptClient(ClientMgrVM one, Guid loginerOID, Guid orgOID)
        {
            Guid key = Guid.Parse(one.OID);
            context.Update<Client_Mgr>(t => t.Client_MgrOID == key, a => new Client_Mgr()
            {
                Name = one.Name,
                IP = one.IP,
                GYLXFK = Guid.Parse(one.GYLXFK),
                Note = one.Note,
                SYS_LAST_UPD = DateTime.Now,
                SYS_LAST_UPD_BY = loginerOID,
                SYS_ORG = orgOID,
            });
            return one.OID;
        }

        /// <summary>
        /// 根据IP修改设备的在线状态和配置
        /// </summary>
        /// <param name="one">修改的数据</param>
        /// <returns></returns>
        public bool UptClientCnf(ClientMgrVM one)
        {
            context.Update<Client_Mgr>(t => t.IP == one.IP, a => new Client_Mgr()
            {
                Online = Convert.ToBoolean(one.Online),
                PCConfig = one.PCConfig
            });
            return true;
        }

        /// <summary>
        /// 删除终端
        /// </summary>
        /// <param name="oid">删除终端的OID列表</param>
        /// <returns></returns>
        public bool DelClient(List<Guid> keys)
        {
            context.Delete<Client_Mgr>(t => keys.Contains(t.Client_MgrOID));
            return true;
        }

        /// <summary>
        /// 根据查询条件得到终端数据
        /// </summary>
        /// <param name="search">查询条件</param>
        /// <returns></returns>
        public PageData GetClientPager(ClientMgrSearch search)
        {
            var where = new LamadaExtention<ClientMgrList>();
            if (!string.IsNullOrEmpty(search.ID))
            {
                where.And("ID", search.ID);
            }
            if (!string.IsNullOrEmpty(search.Name))
            {
                where.And("Name", search.Name);
            }
            if (!string.IsNullOrEmpty(search.IP))
            {
                where.And("IP", search.IP, ExpressionType.Contains);
            }
            if (!string.IsNullOrEmpty(search.GYLXFK))
            {
                where.And("GYLXFK", Guid.Parse(search.GYLXFK));
            }
            if (search.Online != null)
            {
                where.And("Online", search.Online.Value);
            }
            var query = context.JoinQuery<Client_Mgr, Sys_User, B_ProductProcessDetail>((a, b, c) => new object[] {
                JoinType.LeftJoin,a.SYS_LAST_UPD_BY==b.Sys_UserOID,
                JoinType.LeftJoin,a.GYLXFK==c.B_ProductProcessDetailOID
            })
            .Select((a, b, c) => new ClientMgrList
            {
                OID = a.Client_MgrOID,
                GYLXFK = a.GYLXFK,
                GYLXName = c.GXMC,
                ID = a.ID,
                IP = a.IP,
                Name = a.Name,
                Note = a.Note,
                Online = a.Online.Value,
                PCConfig = a.PCConfig,
                WHR = b.EmployeeName
            }).Where(where.GetLambda());
            int total = query.Count();
            var data = query.OrderByDesc(t => t.ID).TakePage(search.pageindex, search.pagesize).ToList();
            PageData pager = PageData.Make(search.pageindex, search.pagesize, total, data);
            return pager;
        }
        /// <summary>
        /// 得到一个终端
        /// </summary>
        /// <param name="key">终端的KEY值</param>
        /// <returns></returns>
        public ClientMgrList GetOneClient(Guid key)
        {
            var query = context.JoinQuery<Client_Mgr, Sys_User, B_ProductProcessDetail>((a, b, c) => new object[] {
                JoinType.LeftJoin,a.SYS_LAST_UPD_BY==b.Sys_UserOID,
                JoinType.LeftJoin,a.GYLXFK==c.B_ProductProcessDetailOID
            })
           .Select((a, b, c) => new ClientMgrList
           {
               OID = a.Client_MgrOID,
               GYLXFK = a.GYLXFK,
               GYLXName = c.GXMC,
               ID = a.ID,
               IP = a.IP,
               Name = a.Name,
               Note = a.Note,
               Online = a.Online.Value,
               PCConfig = a.PCConfig,
               WHR = b.EmployeeName
           }).Where(t => t.OID == key).FirstOrDefault();
            return query;
        }




    }
}
