﻿using Chloe;
using Common.Unit;
using G.Buss.Client.ViewModel;
using G.MES.DataModel;
using G.MES.ViewModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace G.Buss.Client
{
    /// <summary>
    /// 终端版本发布
    /// </summary>
    public class ClientPubSvc : BaseSvc
    {
        /// <summary>
        /// 添加发布记录
        /// </summary>
        /// <param name="data"></param>
        /// <param name="userOID"></param>
        /// <param name="orgOID"></param>
        /// <returns></returns>
        public string AddPub(ClientPubVM data, Guid userOID, Guid orgOID)
        {
            Guid key = Guid.NewGuid();
            Client_Publish pub = new Client_Publish();
            pub.Client_PubOID = key;
            pub.Version = data.Version;
            pub.IsCurrent = false;
            pub.PubDes = data.PubDes;
            pub.FileAddr = data.FileAddr;
            pub.SYS_Created = DateTime.Now;
            pub.SYS_CreatedBy = userOID;
            pub.SYS_LAST_UPD = DateTime.Now;
            pub.SYS_LAST_UPD_BY = userOID;
            pub.SYS_ORG = orgOID;
            context.Insert<Client_Publish>(pub);
        
            return key.ToString();
            
        }
        /// <summary>
        /// 更新发布记录
        /// </summary>
        /// <param name="data"></param>
        /// <param name="userOID"></param>
        /// <param name="orgOID"></param>
        /// <returns></returns>
        public string UptPub(ClientPubVM data, Guid userOID, Guid orgOID)
        {
            Guid key = Guid.Parse(data.OID);
            context.Update<Client_Publish>(t => t.Client_PubOID == key, a => new Client_Publish
            {
                Version = data.Version,
                PubDes = data.PubDes,
                FileAddr = data.FileAddr,
                SYS_LAST_UPD = DateTime.Now,
                SYS_LAST_UPD_BY = userOID,
                SYS_ORG = orgOID
            });
            return data.OID;
        }

        /// <summary>
        /// 设置某一个发布版本为当前版本
        /// </summary>
        /// <param name="key"></param>
        /// <returns></returns>
        public bool SetCurrentPub(Guid key)
        {
            context.Session.BeginTransaction();
            try
            {
                context.Update<Client_Publish>(t => t.IsCurrent == true, a => new Client_Publish { IsCurrent = false });
                context.Update<Client_Publish>(t => t.Client_PubOID == key, a => new Client_Publish { IsCurrent = true });
                context.Session.CommitTransaction();
            }
            catch (Exception ex)
            {
                if (context.Session.IsInTransaction)
                    context.Session.RollbackTransaction();
            }
            return true;
        }


        /// <summary>
        /// 根据条件删除发布记录
        /// </summary>
        /// <param name="keys">发布记录列表</param>
        /// <returns></returns>
        public bool DelPub(List<Guid> keys)
        {
            context.Delete<Client_Publish>(t => keys.Contains(t.Client_PubOID));
            return true;
        }
        /// <summary>
        /// 根据条件查询发布分页数据
        /// </summary>
        /// <param name="search">查询条件</param>
        /// <returns></returns>
        public PageData GetPubPager(ClientPubSearch search)
        {
            var where = new LamadaExtention<ClientPubList>();
            if (!string.IsNullOrEmpty(search.Version))
            {
      
                where.And(t=>t.Version.Contains(search.Version));
            }
            if (search.IsCurrent != null)
            {
               
                where.And(t => t.IsCurrent== search.IsCurrent);
            }
            var query = context.JoinQuery<Client_Publish, Sys_User>((a, b) => new object[] {
                      JoinType.LeftJoin,a.SYS_LAST_UPD_BY==b.Sys_UserOID,
            })
            .Select((a, b) => new ClientPubList
            {
                OID = a.Client_PubOID,
                Version = a.Version,
                FileAddr = a.FileAddr,
                IsCurrent = a.IsCurrent,
                PubDes = a.PubDes,
                WHR = b.EmployeeName,
                PubDate = a.SYS_LAST_UPD
            }).Where(where.GetLambda());
            int totalCount = query.Count();
            var data = query.OrderByDesc(t => t.PubDate).TakePage(search.pageindex, search.pagesize).ToList();
            PageData pager = PageData.Make(search.pageindex, search.pagesize, totalCount, data);
            return pager;
        }

        /// <summary>
        /// 根据key值查询发布记录
        /// </summary>
        /// <param name="key">记录OID</param>
        /// <returns></returns>
        public ClientPubList GetOnePub(Guid key)
        {
            var data = context.JoinQuery<Client_Publish, Sys_User>((a, b) => new object[] {
                      JoinType.LeftJoin,a.SYS_LAST_UPD_BY==b.Sys_UserOID,
            }).Select((a, b) => new ClientPubList
            {
                OID = a.Client_PubOID,
                Version = a.Version,
                FileAddr = a.FileAddr,
                IsCurrent = a.IsCurrent,
                PubDes = a.PubDes,
                WHR = b.EmployeeName,
                PubDate = a.SYS_LAST_UPD
            }).Where(t => t.OID == key).FirstOrDefault();
            return data;
        }

        /// <summary>
        /// 得到当前的版本信息
        /// </summary>
        /// <returns></returns>
        public ClientPubList GetVersion()
        {
            var data = context.JoinQuery<Client_Publish, Sys_User>((a, b) => new object[] {
                      JoinType.LeftJoin,a.SYS_LAST_UPD_BY==b.Sys_UserOID,
            }).Select((a, b) => new ClientPubList
            {
                OID = a.Client_PubOID,
                Version = a.Version,
                FileAddr = a.FileAddr,
                IsCurrent = a.IsCurrent,
                PubDes = a.PubDes,
                WHR = b.EmployeeName,
                PubDate = a.SYS_LAST_UPD
            }).Where(t => t.IsCurrent == true).FirstOrDefault();
            return data;
        }



    }
}
