﻿using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Text;
using NPOI.HPSF;
using NPOI.HSSF.UserModel;
using NPOI.HSSF.Util;
using NPOI.SS.UserModel;
using NPOI.SS.Util;
using NPOI.XSSF.UserModel;

namespace Nyd.Extend.Tool.NPOI
{
    class NPOIHelper
    {
        private HSSFWorkbook hssfworkbook = new HSSFWorkbook();

        private string xlsPath = "";

        public NPOIHelper(string path)
        {
            xlsPath = path;
            using (FileStream file = new FileStream(path, FileMode.Open, FileAccess.Read))
            {
                hssfworkbook = new HSSFWorkbook(file);
            }
        }

        #region 属性

        public string XlsPath
        {
            get { return xlsPath; }
            set { xlsPath = value; }
        }

        #endregion

        #region  方法
        /// <summary>
        /// 创建一个空的Excel文档，指定sheet名
        /// </summary>
        /// <param name="xlspath">excel保存路径,默认为xls后缀名</param>
        /// <param name="sheets">sheet名称</param>
        public void CreateEmptyExcelFile(string xlspath, params string[] sheets)
        {
            InitializeWorkbook();

            if (sheets.Count() > 0)
            {
                for (int i = 0; i < sheets.Count(); i++)
                {
                    hssfworkbook.CreateSheet(sheets[i]);
                }
            }
            else
            {
                hssfworkbook.CreateSheet("sheet1");
            }

            ((HSSFSheet)hssfworkbook.GetSheetAt(0)).AlternativeFormula = false;
            ((HSSFSheet)hssfworkbook.GetSheetAt(0)).AlternativeExpression = false;

            FileStream file = new FileStream(xlspath + ".xls", FileMode.Create);
            hssfworkbook.Write(file);
            file.Close();
        }
        #endregion

        #region 私有方法

        /// <summary>
        /// 设置xls的信息
        /// </summary>
        private void InitializeWorkbook()
        {
            DocumentSummaryInformation dsi = PropertySetFactory.CreateDocumentSummaryInformation();
            dsi.Company = "NPOI Team";
            hssfworkbook.DocumentSummaryInformation = dsi;

            SummaryInformation si = PropertySetFactory.CreateSummaryInformation();
            si.Subject = "NPOI Example";
            hssfworkbook.SummaryInformation = si;
        }

        private void WriteToFile()
        {
            //Write the stream data of workbook to the root directory
            FileStream file = new FileStream(xlsPath, FileMode.Create);
            hssfworkbook.Write(file);
            file.Close();
        }

        #endregion

        /// <summary>
        /// 写入数据，无格式
        /// </summary>
        /// <param name="Sheetindex">sheet索引</param>
        /// <param name="value"></param>
        public void SetCellValuesInXls(int Sheetindex, int RowIndex, int CellIndex, string value)
        {
            InitializeWorkbook();

            ISheet sheet1 = hssfworkbook.GetSheetAt(Sheetindex);

            sheet1.CreateRow(RowIndex).CreateCell(CellIndex).SetCellValue(value);

            WriteToFile();
        }

        /// <summary>
        /// 写入日期格式
        /// </summary>
        /// <param name="Sheetindex"></param>
        /// <param name="RowIndex"></param>
        /// <param name="CellIndex"></param>
        /// <param name="value"></param>
        public void SetDateCellInXls(int Sheetindex, int RowIndex, int CellIndex, string date)
        {
            InitializeWorkbook();

            ISheet sheet = hssfworkbook.GetSheetAt(Sheetindex);
            // Create a row and put some cells in it. Rows are 0 based.
            IRow row = sheet.CreateRow(RowIndex);

            // Create a cell and put a date value in it.  The first cell is not styled as a date.
            ICell cell = row.CreateCell(CellIndex);
            cell.SetCellValue(date);

            ICellStyle cellStyle = hssfworkbook.CreateCellStyle();

            cellStyle.DataFormat = hssfworkbook.CreateDataFormat().GetFormat("yyyy年m月d日");
            cell.CellStyle = cellStyle;

            //ICell cell2 = row.CreateCell(1);
            //cell2.SetCellValue(new DateTime(2008, 5, 5));
            //ICellStyle cellStyle2 = hssfworkbook.CreateCellStyle();
            //IDataFormat format = hssfworkbook.CreateDataFormat();
            //cellStyle2.DataFormat = format.GetFormat("yyyy年m月d日");
            //cell2.CellStyle = cellStyle2;

            //ICell cell3 = row.CreateCell(2);
            //cell3.CellFormula = "DateValue(\"2005-11-11 11:11:11\")";
            //ICellStyle cellStyle3 = hssfworkbook.CreateCellStyle();
            //cellStyle3.DataFormat = HSSFDataFormat.GetBuiltinFormat("m/d/yy h:mm");
            //cell3.CellStyle = cellStyle3;

            WriteToFile();
        }

        /// <summary>
        /// 给表格画边框
        /// </summary>
        /// <param name="Sheetindex"></param>
        /// <param name="RowIndex"></param>
        /// <param name="CellIndex"></param>
        public void SetBorderStyleInXls(int Sheetindex, int RowIndex, int CellIndex)
        {
            InitializeWorkbook();

            ISheet sheet = hssfworkbook.GetSheetAt(Sheetindex);

            // Create a row and put some cells in it. Rows are 0 based.
            IRow row = sheet.GetRow(RowIndex);

            // Create a cell and put a value in it.
            ICell cell = row.GetCell(CellIndex);

            // Style the cell with borders all around.
            ICellStyle style = hssfworkbook.CreateCellStyle();
            style.BorderBottom = BorderStyle.Thin;
            style.BottomBorderColor = HSSFColor.Black.Index;
            style.BorderLeft = BorderStyle.DashDotDot;
            style.LeftBorderColor = HSSFColor.Green.Index;
            style.BorderRight = BorderStyle.Hair;
            style.RightBorderColor = HSSFColor.Blue.Index;
            style.BorderTop = BorderStyle.MediumDashed;
            style.TopBorderColor = HSSFColor.Orange.Index;
            cell.CellStyle = style;
            WriteToFile();
        }

        /// <summary> 
        /// 设置打印区域 
        /// </summary>
        /// <param name="Sheetindex"></param>
        /// <param name="Area">"A5:G20"</param>
        public void SetPrintAreaInXls(int Sheetindex, string Area)
        {
            InitializeWorkbook();

            hssfworkbook.SetPrintArea(Sheetindex, Area);

            WriteToFile();
        }

        /// <summary>
        /// 设置打印格式，默认为A4纸
        /// </summary>
        /// <param name="Sheetindex"></param>
        /// <param name="Area"></param>
        public void SetPrintFormatInXls(int Sheetindex)
        {
            InitializeWorkbook();

            ISheet sheet1 = hssfworkbook.CreateSheet("Sheet1");
            sheet1.SetMargin(MarginType.RightMargin, (double)0.5);
            sheet1.SetMargin(MarginType.TopMargin, (double)0.6);
            sheet1.SetMargin(MarginType.LeftMargin, (double)0.4);
            sheet1.SetMargin(MarginType.BottomMargin, (double)0.3);

            sheet1.PrintSetup.Copies = 3;
            sheet1.PrintSetup.NoColor = true;
            sheet1.PrintSetup.Landscape = true;
            sheet1.PrintSetup.PaperSize = (short)PaperSize.A4;

            sheet1.FitToPage = true;
            sheet1.PrintSetup.FitHeight = 2;
            sheet1.PrintSetup.FitWidth = 3;
            sheet1.IsPrintGridlines = true;

            WriteToFile();
        }

        /// <summary>
        /// 设置表格的宽和高
        /// </summary>
        /// <param name="Sheetindex"></param>
        /// <param name="RowIdex"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        public void SetWidthAndHeightInXls(int Sheetindex, int RowIdex, int CellIndex, int width, short height)
        {
            InitializeWorkbook();

            ISheet sheet1 = hssfworkbook.GetSheetAt(Sheetindex);
            //set the width of columns
            sheet1.SetColumnWidth(CellIndex, width);

            //set the width of height
            sheet1.GetRow(RowIdex).Height = height;

            sheet1.DefaultRowHeightInPoints = 50;

            WriteToFile();
        }

        /// <summary>
        /// 设置单元格对齐方式
        /// </summary>
        /// <param name="Sheetindex"></param>
        /// <param name="RowIdex"></param>
        /// <param name="CellIndex"></param>
        /// <param name="Horizont">水平对齐方式：left,center,right,justify</param>
        /// <param name="Vertical">垂直对齐方式：top,center,buttom,justify</param>
        public void SetAlignmentInXls(int Sheetindex, int RowIdex, int CellIndex, string Horizont, string Vertical)
        {
            InitializeWorkbook();

            ISheet sheet1 = hssfworkbook.GetSheetAt(Sheetindex);

            IRow row = sheet1.GetRow(RowIdex);
            ICellStyle style = hssfworkbook.CreateCellStyle();
            if (Horizont == "left")
            {
                style.Alignment = HorizontalAlignment.Left;
            }
            else if (Horizont == "center")
            {
                style.Alignment = HorizontalAlignment.Center;
            }
            else if (Horizont == "right")
            {
                style.Alignment = HorizontalAlignment.Right;
            }
            else if (Horizont == "justify")
            {
                style.Alignment = HorizontalAlignment.Justify;
            }

            if (Vertical == "top")
            {
                style.VerticalAlignment = VerticalAlignment.Top;
            }
            else if (Vertical == "center")
            {
                style.VerticalAlignment = VerticalAlignment.Center;
            }
            else if (Vertical == "buttom")
            {
                style.VerticalAlignment = VerticalAlignment.Bottom;
            }
            else if (Vertical == "justify")
            {
                style.VerticalAlignment = VerticalAlignment.Justify;
            }
            style.Indention = 3;

            row.GetCell(CellIndex).CellStyle = style;

            WriteToFile();
        }
        /// <summary>
        /// 放大缩小工作簿 根据  sub/den 进行缩放
        /// </summary>
        /// <param name="Sheetindex">要放大的sheet</param>
        /// <param name="sub">比列的分子</param>
        /// <param name="den">比列的分母</param>
        public void ZoomSheet(int Sheetindex, int sub, int den)
        {
            InitializeWorkbook();

            ISheet sheet1 = hssfworkbook.GetSheetAt(Sheetindex);
            sheet1.SetZoom(sub, den);   // 75 percent magnification

            WriteToFile();
        }
        /// <summary>
        /// 在单元格内使用多行存储数据
        /// </summary>
        /// <param name="Sheetindex"></param>
        /// <param name="RowIdex"></param>
        /// <param name="CellIndex"></param>
        /// <param name="rows">使用的行数</param>
        /// <param name="value">在换行的后面加上   \n</param>
        public void UseNewlinesInCellsInXls(int Sheetindex, int RowIdex, int CellIndex, int rows, string value)
        {
            InitializeWorkbook();

            ISheet sheet1 = hssfworkbook.GetSheetAt(Sheetindex);

            //use newlines in cell
            IRow row1 = sheet1.GetRow(RowIdex);
            ICell cell1 = row1.GetCell(CellIndex);

            //to enable newlines you need set a cell styles with wrap=true
            ICellStyle cs = hssfworkbook.CreateCellStyle();
            cs.WrapText = true;
            cell1.CellStyle = cs;

            row1.HeightInPoints = rows * sheet1.DefaultRowHeightInPoints;
            cell1.SetCellValue(value);
            WriteToFile();
        }

        /// <summary>
        /// 单元格使用基础公式
        /// </summary>
        /// <param name="Sheetindex"></param>
        /// <param name="RowIdex"></param>
        /// <param name="CellIndex"></param>
        /// <param name="Formula"></param>
        public void UseBasicFormulaInXls(int Sheetindex, int RowIdex, int CellIndex, string Formula)
        {
            InitializeWorkbook();
            ISheet s1 = hssfworkbook.GetSheetAt(Sheetindex);
            //set A4=A2+A3,set D2=SUM(A2:C2);A5=cos(5)+sin(10)
            s1.GetRow(RowIdex).GetCell(CellIndex).CellFormula = Formula;
            WriteToFile();
        }


        /// <summary>
        /// 冻结行，FreezeRow为要冻结的行
        /// </summary>
        /// <param name="Sheetindex"></param>
        public void SplitAndFreezePanes(int Sheetindex, int FreezeRow)
        {
            InitializeWorkbook();

            ISheet sheet1 = hssfworkbook.GetSheetAt(Sheetindex);

            // Freeze just one row
            sheet1.CreateFreezePane(0, FreezeRow);

            WriteToFile();
        }


        /// <summary>
        /// 缩放指定单元格字体
        /// </summary>
        /// <param name="Sheetindex"></param>
        /// <param name="RowIdex"></param>
        /// <param name="CellIndex"></param>
        public void ShrinkToFitColumnInXls(int Sheetindex, int RowIdex, int CellIndex)
        {
            InitializeWorkbook();

            ISheet sheet = hssfworkbook.GetSheetAt(Sheetindex);
            IRow row = sheet.GetRow(RowIdex);
            //create cell value
            ICell cell1 = row.GetCell(CellIndex);

            ICellStyle cellstyle1 = hssfworkbook.CreateCellStyle();
            cellstyle1.ShrinkToFit = true;
            WriteToFile();
        }


        /// <summary>
        /// 将字体旋转指定角度
        /// </summary>
        /// <param name="Sheetindex"></param>
        /// <param name="RowIdex"></param>
        /// <param name="CellIndex"></param>
        /// <param name="Angle"></param>
        public void RotateTextInXls(int Sheetindex, int RowIdex, int CellIndex, short Angle)
        {
            InitializeWorkbook();

            ISheet sheet1 = hssfworkbook.GetSheetAt(Sheetindex);

            IRow row = sheet1.GetRow(RowIdex);
            //set the style
            ICellStyle style = hssfworkbook.CreateCellStyle();
            style.Rotation = Angle;
            row.GetCell(CellIndex).CellStyle = style;

            WriteToFile();
        }



        /// <summary>
        /// 向单元格中写入数字格式
        /// </summary>
        /// <param name="Sheetindex"></param>
        /// <param name="RowIdex"></param>
        /// <param name="CellIndex"></param>
        /// <param name="type">double，RMB，scentific，percent，phone，ChineseCapital，ChineseDate</param>
        public void NumberFormatInXls(int Sheetindex, int RowIdex, int CellIndex, string type)
        {
            InitializeWorkbook();

            ISheet sheet = hssfworkbook.GetSheetAt(Sheetindex);
            //increase the width of Column A
            sheet.SetColumnWidth(0, 5000);
            //create the format instance
            IDataFormat format = hssfworkbook.CreateDataFormat();

            // Create a row and put some cells in it. Rows are 0 based.
            ICell cell = sheet.GetRow(RowIdex).GetCell(CellIndex);
            ICellStyle cellStyle = hssfworkbook.CreateCellStyle();

            if (type == "double")
            {
                cellStyle.DataFormat = HSSFDataFormat.GetBuiltinFormat("0.00");
            }
            else if (type == "RMB")
            {
                cellStyle.DataFormat = format.GetFormat("¥#,##0");
            }
            else if (type == "scentific")
            {
                cellStyle.DataFormat = HSSFDataFormat.GetBuiltinFormat("0.00E+00");
            }
            else if (type == "percent")
            {
                cellStyle.DataFormat = HSSFDataFormat.GetBuiltinFormat("0.00%");
            }
            else if (type == "phone")//phone number format - "021-65881234"
            {
                cellStyle.DataFormat = format.GetFormat("000-00000000");
            }
            else if (type == "ChineseCapital") //Chinese capitalized character number - 壹贰叁 元
            {
                cellStyle.DataFormat = format.GetFormat("[DbNum2][$-804]0 元");
            }
            else if (type == "ChineseDate")
            {
                cellStyle.DataFormat = format.GetFormat("yyyy年m月d日");
            }
            cell.CellStyle = cellStyle;

            WriteToFile();
        }


        /// <summary>
        /// 将一个单元格赋予两个表格的乘积
        /// </summary>
        /// <param name="Sheetindex"></param>
        /// <param name="RowIdex">要赋值的单元格行索引</param>
        /// <param name="CellIndex">要赋值的单元格列索引</param>
        /// <param name="targateRow1">第一个单元格的行</param>
        /// <param name="targateCell1">第一个单元格的列</param>
        /// <param name="targateRow2">第二个单元格的行</param>
        /// <param name="targateCell2">第二个单元格的列</param>
        public void MultplicationTableInXls(int Sheetindex, int RowIdex, int CellIndex, int targateRow1, int targateCell1,
            int targateRow2, int targateCell2)
        {
            InitializeWorkbook();

            ISheet sheet1 = hssfworkbook.GetSheetAt(Sheetindex);

            IRow row = sheet1.GetRow(RowIdex);

            string formula = GetCellPosition(targateRow1, targateCell1) + "*" + GetCellPosition(targateRow2, targateCell2);

            row.CreateCell(CellIndex).CellFormula = formula;

            WriteToFile();
        }

        public string GetCellPosition(int row, int col)
        {
            col = Convert.ToInt32('A') + col;
            row = row + 1;
            return ((char)col) + row.ToString();
        }
        /// <summary>
        /// 合并单元格
        /// </summary>
        /// <param name="Sheetindex"></param>
        /// <param name="firstRowIdex">起始单元格</param>
        /// <param name="firstCellIndex"></param>
        /// <param name="lastRowIdex">结束单元格</param>
        /// <param name="lastCellIndex"></param>
        public void MergeCellsInXls(int Sheetindex, int firstRowIdex, int firstCellIndex, int lastRowIdex, int lastCellIndex)
        {
            InitializeWorkbook();

            ISheet sheet = hssfworkbook.GetSheetAt(Sheetindex);

            sheet.AddMergedRegion(new CellRangeAddress(0, 0, 0, 5));

            CellRangeAddress region = new CellRangeAddress(firstRowIdex, lastRowIdex, firstCellIndex, lastCellIndex);

            sheet.AddMergedRegion(region);

            WriteToFile();
        }



        public string GetCellValue(int sheetIndex, int rowIndex, int cellIndex)
        {
            InitializeWorkbook();

            ISheet sheet1 = hssfworkbook.GetSheetAt(sheetIndex);

            ICell cell = sheet1.GetRow(rowIndex).GetCell(cellIndex);

            return cell.StringCellValue;

        }



        #region 导入导出
        /* 
         * DataTable table = new DataTable();     
         * MemoryStream ms = DataTableRenderToExcel.RenderDataTableToExcel(table) as MemoryStream;
         * Response.AddHeader("Content-Disposition", string.Format("attachment; filename=Download.xls"));
         * Response.BinaryWrite(ms.ToArray());
         * ms.Close();
         * ms.Dispose();
       
         * if (this.fuUpload.HasFile)
         * {
         *       DataTable table = DataTableRenderToExcel.RenderDataTableFromExcel(this.fuUpload.FileContent, 1, 0);
         *       this.gvExcel.DataSource = table;
         *       this.gvExcel.DataBind();
         *  }
         */
        /// <summary>
        /// Datatable导出Excel
        /// </summary>
        /// <param name="SourceTable"></param>
        /// <returns></returns>
        private static Stream RenderDataTableToExcel(DataTable SourceTable)
        {
            HSSFWorkbook workbook = new HSSFWorkbook();
            MemoryStream ms = new MemoryStream();
            ISheet sheet = workbook.CreateSheet();
            IRow headerRow = sheet.CreateRow(0);

            // handling header.
            foreach (DataColumn column in SourceTable.Columns)
                headerRow.CreateCell(column.Ordinal).SetCellValue(column.ColumnName);

            // handling value.
            int rowIndex = 1;

            foreach (DataRow row in SourceTable.Rows)
            {
                IRow dataRow = sheet.CreateRow(rowIndex);
                foreach (DataColumn column in SourceTable.Columns)
                {
                    dataRow.CreateCell(column.Ordinal).SetCellValue(row[column].ToString());
                }

                rowIndex++;
            }

            workbook.Write(ms);
            ms.Flush();
            ms.Position = 0;

            sheet = null;
            headerRow = null;
            workbook = null;

            return ms;
        }
        /// <summary>
        /// Datatable导出Excel
        /// </summary>
        /// <param name="SourceTable"></param>
        /// <param name="FileName"></param>
        public static void RenderDataTableToExcel(DataTable SourceTable, string FileName)
        {
            MemoryStream ms = RenderDataTableToExcel(SourceTable) as MemoryStream;
            FileStream fs = new FileStream(FileName, FileMode.Create, FileAccess.Write);
            byte[] data = ms.ToArray();
            fs.Write(data, 0, data.Length);
            fs.Flush();
            fs.Close();
            data = null;
            ms = null;
            fs = null;
        }
        /// <summary>
        /// 导出excel为Datatable
        /// </summary>
        /// <param name="ExcelFileStream"></param>
        /// <param name="SheetName"></param>
        /// <param name="HeaderRowIndex"></param>
        /// <returns></returns>
        public static DataTable RenderDataTableFromExcel(Stream ExcelFileStream, string SheetName, int HeaderRowIndex)
        {
            HSSFWorkbook workbook = new HSSFWorkbook(ExcelFileStream);
            ISheet sheet = workbook.GetSheet(SheetName);

            DataTable table = new DataTable();

            IRow headerRow = sheet.GetRow(HeaderRowIndex);
            int cellCount = headerRow.LastCellNum;

            for (int i = headerRow.FirstCellNum; i < cellCount; i++)
            {
                DataColumn column = new DataColumn(headerRow.GetCell(i).StringCellValue);
                table.Columns.Add(column);
            }

            int rowCount = sheet.LastRowNum;

            for (int i = (sheet.FirstRowNum + 1); i < sheet.LastRowNum; i++)
            {
                IRow row = sheet.GetRow(i);
                DataRow dataRow = table.NewRow();

                for (int j = row.FirstCellNum; j < cellCount; j++)
                    dataRow[j] = row.GetCell(j).ToString();
            }

            ExcelFileStream.Close();
            workbook = null;
            sheet = null;
            return table;
        }
        /// <summary>
        /// 将Excel转换为Datatable
        /// </summary>
        /// <param name="ExcelFileStream"></param>
        /// <param name="SheetIndex"></param>
        /// <param name="HeaderRowIndex"></param>
        /// <returns></returns>
        public static DataTable RenderDataTableFromExcel(Stream ExcelFileStream, int SheetIndex, int HeaderRowIndex)
        {
            XSSFWorkbook workbook = new XSSFWorkbook(ExcelFileStream);
            ISheet sheet = workbook.GetSheetAt(SheetIndex);

            DataTable table = new DataTable();

            IRow headerRow = sheet.GetRow(HeaderRowIndex);
            int cellCount = headerRow.LastCellNum;

            for (int i = headerRow.FirstCellNum; i < cellCount; i++)
            {
                DataColumn column = new DataColumn(headerRow.GetCell(i).StringCellValue);
                table.Columns.Add(column);
            }

            int rowCount = sheet.LastRowNum;

            for (int i = (sheet.FirstRowNum + 1); i < sheet.LastRowNum; i++)
            {
                IRow row = sheet.GetRow(i);
                DataRow dataRow = table.NewRow();

                for (int j = row.FirstCellNum; j < cellCount; j++)
                {
                    if (row.GetCell(j) != null)
                        dataRow[j] = row.GetCell(j).ToString();
                }

                table.Rows.Add(dataRow);
            }

            ExcelFileStream.Close();
            workbook = null;
            sheet = null;
            return table;
        }
        #endregion
    }
}
