﻿using G.Buss.BaseInfo.Services.ExportWordSVC;
using G.Buss.BaseInfo.ViewModel.ClientInfoModel;
using G.Buss.BaseInfo.ViewModel.CodeTableModel;
using G.Buss.BaseInfo.ViewModel.GeneCorrelationModel;
using G.Buss.BaseInfo.ViewModel.MISReportModel;
using G.Buss.BaseInfo.ViewModel.SampleControlModel;
using G.Buss.BaseInfo.ViewModel.TMBGradeRequestModel;
using G.MES.DataModel;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WordByDocx.Word
{
    /// <summary>
    /// 自由操作固定的数据获取与释放
    /// </summary>
    public class GetData : IDisposable
    {
        private static readonly ReportDataSvc svc = new ReportDataSvc();
        private static List<TMBTable> TMBs = null;
        private static List<MISReportQueryModel> MSIs = null;
        private static string TargetDrugReferences = string.Empty;
        private static string ChemicalMedicineReferences = string.Empty;
        /// <summary>
        /// 获取受试者信息
        /// </summary>
        /// <param name="guid"></param>
        /// <returns></returns>
        public static ClientInfoDetailModel GetUserInfo(Guid guid)
        {
            return svc.GetClirntInfo(guid);
        }


        /// <summary>
        /// 获取样本质控情况
        /// </summary>
        public static List<SampleControlQueryModel> GetSampleQuality() => svc.GetSampleQuality();

        /// <summary>
        /// 获取检测结果
        /// </summary>
        /// <returns></returns>
        public static DataTable GetSummary()
        {
            return svc.GetCheckResult();
        }

        /// <summary>
        /// 获取化疗结果
        /// </summary>
        /// <returns></returns>
        public static DataTable GetChemotherapy()
        {
            return svc.GetChemotherapyDrugs();
        }

        /// <summary>
        /// 获取TMB表格(这里不考虑线程的问题)
        /// </summary>
        public static List<TMBTable> GetTMBTable()
        {
            if (TMBs == null)
            {
                TMBs = svc.GetTMBLevel();
            }
            return TMBs;
        }

        /// <summary>
        /// 获取TMB表格(这里不考虑线程的问题)
        /// </summary>
        public static List<MISReportQueryModel> GetMSITable()
        {
            if (MSIs == null)
            {
                MSIs = svc.GetMIS();
            }
            return MSIs;
        }
        /// <summary>
        /// 获取基因列表
        /// </summary>
        /// <returns></returns>
        public static List<GeneCorrelationQueryModel> GetGeneList()
        {
            return svc.GetGeneList();
        }

        /// <summary>
        /// 获取基因类型列表
        /// </summary>
        /// <returns></returns>
        public static List<SonCodeTableResponseModel> GetGeneTypeList()
        {
            return svc.GetGeneTypeList();
        }

        /// <summary>
        /// 获取靶药文献
        /// </summary>
        /// <returns></returns>
        public static string GetTargetDrugReferences()
        {
            if (string.IsNullOrEmpty(TargetDrugReferences))
            {
                TargetDrugReferences = svc.GetTargetDrugReferences();
            }
            return TargetDrugReferences;
        }
        /// <summary>
        /// 获取化药文献
        /// </summary>
        /// <returns></returns>
        public static string GetChemicalMedicineReferences()
        {
            if (string.IsNullOrEmpty(ChemicalMedicineReferences))
            {
                ChemicalMedicineReferences = svc.GetChemicalMdicineReferences();
            }
            return ChemicalMedicineReferences;
        }


        /// <summary>
        /// 暂未做处理
        /// </summary>
        public void Dispose()
        {

        }
    }
}
